@@ -1,14 +1,12 @@
 /*
  * blocklist.c
- * Copyright (C) 1998,1999 A.J. van Os
+ * Copyright (C) 1998-2000 A.J. van Os; Released under GPL
  *
  * Description:
  * Build, read and destroy a list of Word text blocks
  */
 
 #include <stdlib.h>
-#include <string.h>
-#include <limits.h>
 #include "antiword.h"
 
 
@@ -88,33 +86,32 @@ bAdd2TextBlockList(text_block_type *pTextBlock)
 	list_mem_type	*pListMember;
 
 	fail(pTextBlock == NULL);
-	fail(pTextBlock->iFileOffset < 0);
-	fail(pTextBlock->iTextOffset < 0);
-	fail(pTextBlock->iLength <= 0);
-	fail(pTextBlock->bUsesUnicode != !!pTextBlock->bUsesUnicode);
-	fail(pTextBlock->bUsesUnicode && odd(pTextBlock->iLength));
+	fail(pTextBlock->lFileOffset < 0);
+	fail(pTextBlock->lTextOffset < 0);
+	fail(pTextBlock->tLength == 0);
+	fail(pTextBlock->bUsesUnicode && odd(pTextBlock->tLength));
 
 	NO_DBG_MSG("bAdd2TextBlockList");
-	NO_DBG_HEX(pTextBlock->iFileOffset);
-	NO_DBG_HEX(pTextBlock->iTextOffset);
-	NO_DBG_HEX(pTextBlock->iLength);
+	NO_DBG_HEX(pTextBlock->lFileOffset);
+	NO_DBG_HEX(pTextBlock->lTextOffset);
+	NO_DBG_HEX(pTextBlock->tLength);
 	NO_DBG_DEC(pTextBlock->bUsesUnicode);
 
-	if (pTextBlock->iFileOffset < 0 ||
-	    pTextBlock->iTextOffset < 0 ||
-	    pTextBlock->iLength <= 0) {
+	if (pTextBlock->lFileOffset < 0 ||
+	    pTextBlock->lTextOffset < 0 ||
+	    pTextBlock->tLength == 0) {
 		werr(0, "Software (textblock) error");
 		return FALSE;
 	}
 	/* Check for continuous blocks of the same character size */
 	if (pBlockLast != NULL &&
-	    pBlockLast->tInfo.iFileOffset +
-	     pBlockLast->tInfo.iLength == pTextBlock->iFileOffset &&
-	    pBlockLast->tInfo.iTextOffset +
-	     pBlockLast->tInfo.iLength == pTextBlock->iTextOffset &&
+	    pBlockLast->tInfo.lFileOffset +
+	     (long)pBlockLast->tInfo.tLength == pTextBlock->lFileOffset &&
+	    pBlockLast->tInfo.lTextOffset +
+	     (long)pBlockLast->tInfo.tLength == pTextBlock->lTextOffset &&
 	    pBlockLast->tInfo.bUsesUnicode == pTextBlock->bUsesUnicode) {
 		/* These are continous blocks */
-		pBlockLast->tInfo.iLength += pTextBlock->iLength;
+		pBlockLast->tInfo.tLength += pTextBlock->tLength;
 		return TRUE;
 	}
 	/* Make a new block */
@@ -144,38 +141,38 @@ bAdd2TextBlockList(text_block_type *pTextBlock)
  * length of the blocks are in bytes.
  */
 void
-vSplitBlockList(int iTextLen, int iFootnoteLen,
-		int iUnused1Len, int iEndnoteLen, int iUnused2Len,
-		BOOL bMustExtend)
+vSplitBlockList(size_t tTextLen, size_t tFootnoteLen, size_t tUnused1Len,
+	size_t tEndnoteLen, size_t tUnused2Len, BOOL bMustExtend)
 {
 	list_mem_type	*apAnchors[5];
 	list_mem_type	*pGarbageAnchor, *pCurr, *pNext;
 	int		iIndex, iCharsToGo, iBytesTooFar;
 #if defined(DEBUG)
-	int		iTotal;
+	size_t		tTotal;
 #endif /* DEBUG */
 
 	DBG_MSG("vSplitBlockList");
 
 /* Text block list */
-	if (iTextLen > 0) {
+	pCurr = NULL;
+	iCharsToGo = (int)tTextLen;
+	iBytesTooFar = -1;
+	if (tTextLen != 0) {
 		DBG_MSG("Text block list");
-		DBG_DEC(iTextLen);
-		iCharsToGo = iTextLen;
-		iBytesTooFar = -1;
+		DBG_DEC(tTextLen);
 		for (pCurr = pTextAnchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			DBG_DEC(pCurr->tInfo.iLength);
-			fail(pCurr->tInfo.iLength <= 0);
+			NO_DBG_DEC(pCurr->tInfo.tLength);
+			fail(pCurr->tInfo.tLength == 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.iLength));
-				iCharsToGo -= pCurr->tInfo.iLength / 2;
+				fail(odd(pCurr->tInfo.tLength));
+				iCharsToGo -= (int)(pCurr->tInfo.tLength / 2);
 				if (iCharsToGo < 0) {
 					iBytesTooFar = -2 * iCharsToGo;
 				}
 			} else {
-				iCharsToGo -= pCurr->tInfo.iLength;
+				iCharsToGo -= (int)pCurr->tInfo.tLength;
 				if (iCharsToGo < 0) {
 					iBytesTooFar = -iCharsToGo;
 				}
@@ -186,7 +183,7 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 		}
 	}
 /* Split the list */
-	if (iTextLen <= 0) {
+	if (tTextLen == 0) {
 		/* Empty text blocks list */
 		pFootAnchor = pTextAnchor;
 		pTextAnchor = NULL;
@@ -202,44 +199,45 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 		DBG_DEC(iBytesTooFar);
 		fail(iBytesTooFar <= 0);
 		pFootAnchor = xmalloc(sizeof(list_mem_type));
-		DBG_HEX(pCurr->tInfo.iFileOffset);
-		pFootAnchor->tInfo.iFileOffset =
-				pCurr->tInfo.iFileOffset +
-				pCurr->tInfo.iLength -
+		DBG_HEX(pCurr->tInfo.lFileOffset);
+		pFootAnchor->tInfo.lFileOffset =
+				pCurr->tInfo.lFileOffset +
+				(long)pCurr->tInfo.tLength -
 				iBytesTooFar;
-		DBG_HEX(pFootAnchor->tInfo.iFileOffset);
-		DBG_HEX(pCurr->tInfo.iTextOffset);
-		pFootAnchor->tInfo.iTextOffset =
-				pCurr->tInfo.iTextOffset +
-				pCurr->tInfo.iLength -
+		DBG_HEX(pFootAnchor->tInfo.lFileOffset);
+		DBG_HEX(pCurr->tInfo.lTextOffset);
+		pFootAnchor->tInfo.lTextOffset =
+				pCurr->tInfo.lTextOffset +
+				(long)pCurr->tInfo.tLength -
 				iBytesTooFar;
-		DBG_HEX(pFootAnchor->tInfo.iTextOffset);
-		pFootAnchor->tInfo.iLength = iBytesTooFar;
-		pCurr->tInfo.iLength -= iBytesTooFar;
+		DBG_HEX(pFootAnchor->tInfo.lTextOffset);
+		pFootAnchor->tInfo.tLength = (size_t)iBytesTooFar;
+		pCurr->tInfo.tLength -= (size_t)iBytesTooFar;
 		pFootAnchor->tInfo.bUsesUnicode = pCurr->tInfo.bUsesUnicode;
 		/* Move the integral number of footnote blocks */
 		pFootAnchor->pNext = pCurr->pNext;
 		pCurr->pNext = NULL;
 	}
 /* Footnote block list */
-	if (iFootnoteLen > 0) {
+	pCurr = NULL;
+	iCharsToGo = (int)tFootnoteLen;
+	iBytesTooFar = -1;
+	if (tFootnoteLen != 0) {
 		DBG_MSG("Footnote block list");
-		DBG_DEC(iFootnoteLen);
-		iCharsToGo = iFootnoteLen;
-		iBytesTooFar = -1;
+		DBG_DEC(tFootnoteLen);
 		for (pCurr = pFootAnchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			DBG_DEC(pCurr->tInfo.iLength);
-			fail(pCurr->tInfo.iLength <= 0);
+			DBG_DEC(pCurr->tInfo.tLength);
+			fail(pCurr->tInfo.tLength == 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.iLength));
-				iCharsToGo -= pCurr->tInfo.iLength / 2;
+				fail(odd(pCurr->tInfo.tLength));
+				iCharsToGo -= (int)(pCurr->tInfo.tLength / 2);
 				if (iCharsToGo < 0) {
 					iBytesTooFar = -2 * iCharsToGo;
 				}
 			} else {
-				iCharsToGo -= pCurr->tInfo.iLength;
+				iCharsToGo -= (int)pCurr->tInfo.tLength;
 				if (iCharsToGo < 0) {
 					iBytesTooFar = -iCharsToGo;
 				}
@@ -250,7 +248,7 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 		}
 	}
 /* Split the list */
-	if (iFootnoteLen <= 0) {
+	if (tFootnoteLen == 0) {
 		/* Empty footnote list */
 		pUnused1Anchor = pFootAnchor;
 		pFootAnchor = NULL;
@@ -266,20 +264,20 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 		DBG_DEC(iBytesTooFar);
 		fail(iBytesTooFar <= 0);
 		pUnused1Anchor = xmalloc(sizeof(list_mem_type));
-		DBG_HEX(pCurr->tInfo.iFileOffset);
-		pUnused1Anchor->tInfo.iFileOffset =
-				pCurr->tInfo.iFileOffset +
-				pCurr->tInfo.iLength -
+		DBG_HEX(pCurr->tInfo.lFileOffset);
+		pUnused1Anchor->tInfo.lFileOffset =
+				pCurr->tInfo.lFileOffset +
+				(long)pCurr->tInfo.tLength -
 				iBytesTooFar;
-		DBG_HEX(pUnused1Anchor->tInfo.iFileOffset);
-		DBG_HEX(pCurr->tInfo.iTextOffset);
-		pUnused1Anchor->tInfo.iTextOffset =
-				pCurr->tInfo.iTextOffset +
-				pCurr->tInfo.iLength -
+		DBG_HEX(pUnused1Anchor->tInfo.lFileOffset);
+		DBG_HEX(pCurr->tInfo.lTextOffset);
+		pUnused1Anchor->tInfo.lTextOffset =
+				pCurr->tInfo.lTextOffset +
+				(long)pCurr->tInfo.tLength -
 				iBytesTooFar;
-		DBG_HEX(pUnused1Anchor->tInfo.iTextOffset);
-		pUnused1Anchor->tInfo.iLength = iBytesTooFar;
-		pCurr->tInfo.iLength -= iBytesTooFar;
+		DBG_HEX(pUnused1Anchor->tInfo.lTextOffset);
+		pUnused1Anchor->tInfo.tLength = (size_t)iBytesTooFar;
+		pCurr->tInfo.tLength -= (size_t)iBytesTooFar;
 		pUnused1Anchor->tInfo.bUsesUnicode =
 				pCurr->tInfo.bUsesUnicode;
 		/* Move the integral number of unused1 blocks */
@@ -287,24 +285,25 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 		pCurr->pNext = NULL;
 	}
 /* Unused1 block list */
-	if (iUnused1Len > 0) {
+	pCurr = NULL;
+	iCharsToGo = (int)tUnused1Len;
+	iBytesTooFar = -1;
+	if (tUnused1Len != 0) {
 		DBG_MSG("Unused1 block list");
-		DBG_DEC(iUnused1Len);
-		iCharsToGo = iUnused1Len;
-		iBytesTooFar = -1;
+		DBG_DEC(tUnused1Len);
 		for (pCurr = pUnused1Anchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			DBG_DEC(pCurr->tInfo.iLength);
-			fail(pCurr->tInfo.iLength <= 0);
+			DBG_DEC(pCurr->tInfo.tLength);
+			fail(pCurr->tInfo.tLength == 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.iLength));
-				iCharsToGo -= pCurr->tInfo.iLength / 2;
+				fail(odd(pCurr->tInfo.tLength));
+				iCharsToGo -= (int)(pCurr->tInfo.tLength / 2);
 				if (iCharsToGo < 0) {
 					iBytesTooFar = -2 * iCharsToGo;
 				}
 			} else {
-				iCharsToGo -= pCurr->tInfo.iLength;
+				iCharsToGo -= (int)pCurr->tInfo.tLength;
 				if (iCharsToGo < 0) {
 					iBytesTooFar = -iCharsToGo;
 				}
@@ -315,7 +314,7 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 		}
 	}
 /* Split the list */
-	if (iUnused1Len <= 0) {
+	if (tUnused1Len == 0) {
 		/* Empty unused1 list */
 		pEndAnchor = pUnused1Anchor;
 		pUnused1Anchor = NULL;
@@ -331,44 +330,45 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 		DBG_DEC(iBytesTooFar);
 		fail(iBytesTooFar <= 0);
 		pEndAnchor = xmalloc(sizeof(list_mem_type));
-		DBG_HEX(pCurr->tInfo.iFileOffset);
-		pEndAnchor->tInfo.iFileOffset =
-				pCurr->tInfo.iFileOffset +
-				pCurr->tInfo.iLength -
+		DBG_HEX(pCurr->tInfo.lFileOffset);
+		pEndAnchor->tInfo.lFileOffset =
+				pCurr->tInfo.lFileOffset +
+				(long)pCurr->tInfo.tLength -
 				iBytesTooFar;
-		DBG_HEX(pEndAnchor->tInfo.iFileOffset);
-		DBG_HEX(pCurr->tInfo.iTextOffset);
-		pEndAnchor->tInfo.iTextOffset =
-				pCurr->tInfo.iTextOffset +
-				pCurr->tInfo.iLength -
+		DBG_HEX(pEndAnchor->tInfo.lFileOffset);
+		DBG_HEX(pCurr->tInfo.lTextOffset);
+		pEndAnchor->tInfo.lTextOffset =
+				pCurr->tInfo.lTextOffset +
+				(long)pCurr->tInfo.tLength -
 				iBytesTooFar;
-		DBG_HEX(pEndAnchor->tInfo.iTextOffset);
-		pEndAnchor->tInfo.iLength = iBytesTooFar;
-		pCurr->tInfo.iLength -= iBytesTooFar;
+		DBG_HEX(pEndAnchor->tInfo.lTextOffset);
+		pEndAnchor->tInfo.tLength = (size_t)iBytesTooFar;
+		pCurr->tInfo.tLength -= (size_t)iBytesTooFar;
 		pEndAnchor->tInfo.bUsesUnicode = pCurr->tInfo.bUsesUnicode;
 		/* Move the integral number of endnote blocks */
 		pEndAnchor->pNext = pCurr->pNext;
 		pCurr->pNext = NULL;
 	}
 /* Endnote block list */
-	if (iEndnoteLen > 0) {
+	pCurr = NULL;
+	iCharsToGo = (int)tEndnoteLen;
+	iBytesTooFar = -1;
+	if (tEndnoteLen != 0) {
 		DBG_MSG("Endnote block list");
-		DBG_DEC(iEndnoteLen);
-		iCharsToGo = iEndnoteLen;
-		iBytesTooFar = -1;
+		DBG_DEC(tEndnoteLen);
 		for (pCurr = pEndAnchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			DBG_DEC(pCurr->tInfo.iLength);
-			fail(pCurr->tInfo.iLength <= 0);
+			DBG_DEC(pCurr->tInfo.tLength);
+			fail(pCurr->tInfo.tLength == 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.iLength));
-				iCharsToGo -= pCurr->tInfo.iLength / 2;
+				fail(odd(pCurr->tInfo.tLength));
+				iCharsToGo -= (int)(pCurr->tInfo.tLength / 2);
 				if (iCharsToGo <= 0) {
 					iBytesTooFar = -2 * iCharsToGo;
 				}
 			} else {
-				iCharsToGo -= pCurr->tInfo.iLength;
+				iCharsToGo -= (int)pCurr->tInfo.tLength;
 				if (iCharsToGo <= 0) {
 					iBytesTooFar = -iCharsToGo;
 				}
@@ -379,7 +379,7 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 		}
 	}
 /* Split the list */
-	if (iEndnoteLen <= 0) {
+	if (tEndnoteLen == 0) {
 		/* Empty endnote list */
 		pUnused2Anchor = pEndAnchor;
 		pEndAnchor = NULL;
@@ -395,20 +395,20 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 		DBG_DEC(iBytesTooFar);
 		fail(iBytesTooFar <= 0);
 		pUnused2Anchor = xmalloc(sizeof(list_mem_type));
-		DBG_HEX(pCurr->tInfo.iFileOffset);
-		pUnused2Anchor->tInfo.iFileOffset =
-				pCurr->tInfo.iFileOffset +
-				pCurr->tInfo.iLength -
+		DBG_HEX(pCurr->tInfo.lFileOffset);
+		pUnused2Anchor->tInfo.lFileOffset =
+				pCurr->tInfo.lFileOffset +
+				(long)pCurr->tInfo.tLength -
 				iBytesTooFar;
-		DBG_HEX(pUnused2Anchor->tInfo.iFileOffset);
-		DBG_HEX(pCurr->tInfo.iTextOffset);
-		pUnused2Anchor->tInfo.iTextOffset =
-				pCurr->tInfo.iTextOffset +
-				pCurr->tInfo.iLength -
+		DBG_HEX(pUnused2Anchor->tInfo.lFileOffset);
+		DBG_HEX(pCurr->tInfo.lTextOffset);
+		pUnused2Anchor->tInfo.lTextOffset =
+				pCurr->tInfo.lTextOffset +
+				(long)pCurr->tInfo.tLength -
 				iBytesTooFar;
-		DBG_HEX(pUnused2Anchor->tInfo.iTextOffset);
-		pUnused2Anchor->tInfo.iLength = iBytesTooFar;
-		pCurr->tInfo.iLength -= iBytesTooFar;
+		DBG_HEX(pUnused2Anchor->tInfo.lTextOffset);
+		pUnused2Anchor->tInfo.tLength = (size_t)iBytesTooFar;
+		pCurr->tInfo.tLength -= (size_t)iBytesTooFar;
 		pUnused2Anchor->tInfo.bUsesUnicode =
 				pCurr->tInfo.bUsesUnicode;
 		/* Move the integral number of unused2 blocks */
@@ -416,24 +416,25 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 		pCurr->pNext = NULL;
 	}
 /* Unused2 block list */
-	if (iUnused2Len > 0) {
+	pCurr = NULL;
+	iCharsToGo = (int)tUnused2Len;
+	iBytesTooFar = -1;
+	if (tUnused2Len != 0) {
 		DBG_MSG("Unused2 block list");
-		DBG_DEC(iUnused2Len);
-		iCharsToGo = iUnused2Len;
-		iBytesTooFar = -1;
+		DBG_DEC(tUnused2Len);
 		for (pCurr = pUnused2Anchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			DBG_DEC(pCurr->tInfo.iLength);
-			fail(pCurr->tInfo.iLength <= 0);
+			DBG_DEC(pCurr->tInfo.tLength);
+			fail(pCurr->tInfo.tLength == 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.iLength));
-				iCharsToGo -= pCurr->tInfo.iLength / 2;
+				fail(odd(pCurr->tInfo.tLength));
+				iCharsToGo -= (int)(pCurr->tInfo.tLength / 2);
 				if (iCharsToGo < 0) {
 					iBytesTooFar = -2 * iCharsToGo;
 				}
 			} else {
-				iCharsToGo -= pCurr->tInfo.iLength;
+				iCharsToGo -= (int)pCurr->tInfo.tLength;
 				if (iCharsToGo < 0) {
 					iBytesTooFar = -iCharsToGo;
 				}
@@ -443,7 +444,8 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 			}
 		}
 	}
-	if (iUnused2Len <= 0) {
+/* Split the list */
+	if (tUnused2Len == 0) {
 		/* Empty unused2 list */
 		pGarbageAnchor = pUnused2Anchor;
 		pUnused2Anchor = NULL;
@@ -457,7 +459,7 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 		/* Reduce the part-endnote block */
 		DBG_DEC(iBytesTooFar);
 		fail(iBytesTooFar <= 0);
-		pCurr->tInfo.iLength -= iBytesTooFar;
+		pCurr->tInfo.tLength -= (size_t)iBytesTooFar;
 		/* Move the integral number of garbage blocks */
 		pGarbageAnchor = pCurr->pNext;
 		pCurr->pNext = NULL;
@@ -465,8 +467,8 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 /* Free the garbage block list, this should never be needed */
 	pCurr = pGarbageAnchor;
 	while (pCurr != NULL) {
-		DBG_HEX(pCurr->tInfo.iFileOffset);
-		DBG_DEC(pCurr->tInfo.iLength);
+		DBG_HEX(pCurr->tInfo.lFileOffset);
+		DBG_DEC(pCurr->tInfo.tLength);
 		pNext = pCurr->pNext;
 		pCurr = xfree(pCurr);
 		pCurr = pNext;
@@ -474,94 +476,94 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 
 #if defined(DEBUG)
 	/* Check the number of bytes in the block lists */
-	iTotal = 0;
+	tTotal = 0;
 	for (pCurr = pTextAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
-		NO_DBG_HEX(pCurr->tInfo.iFileOffset);
-		NO_DBG_HEX(pCurr->tInfo.iTextOffset);
-		NO_DBG_DEC(pCurr->tInfo.iLength);
-		fail(pCurr->tInfo.iLength <= 0);
+		NO_DBG_HEX(pCurr->tInfo.lFileOffset);
+		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
+		NO_DBG_DEC(pCurr->tInfo.tLength);
+		fail(pCurr->tInfo.tLength == 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.iLength));
-			iTotal += pCurr->tInfo.iLength / 2;
+			fail(odd(pCurr->tInfo.tLength));
+			tTotal += pCurr->tInfo.tLength / 2;
 		} else {
-			iTotal += pCurr->tInfo.iLength;
+			tTotal += pCurr->tInfo.tLength;
 		}
 	}
-	DBG_DEC(iTotal);
-	if (iTotal != iTextLen) {
-		DBG_DEC(iTextLen);
+	DBG_DEC(tTotal);
+	if (tTotal != tTextLen) {
+		DBG_DEC(tTextLen);
 		werr(1, "Software error (Text)");
 	}
-	iTotal = 0;
+	tTotal = 0;
 	for (pCurr = pFootAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
-		DBG_HEX(pCurr->tInfo.iFileOffset);
-		NO_DBG_HEX(pCurr->tInfo.iTextOffset);
-		DBG_DEC(pCurr->tInfo.iLength);
-		fail(pCurr->tInfo.iLength <= 0);
+		DBG_HEX(pCurr->tInfo.lFileOffset);
+		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
+		DBG_DEC(pCurr->tInfo.tLength);
+		fail(pCurr->tInfo.tLength == 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.iLength));
-			iTotal += pCurr->tInfo.iLength / 2;
+			fail(odd(pCurr->tInfo.tLength));
+			tTotal += pCurr->tInfo.tLength / 2;
 		} else {
-			iTotal += pCurr->tInfo.iLength;
+			tTotal += pCurr->tInfo.tLength;
 		}
 	}
-	DBG_DEC(iTotal);
-	if (iTotal != iFootnoteLen) {
-		DBG_DEC(iFootnoteLen);
+	DBG_DEC(tTotal);
+	if (tTotal != tFootnoteLen) {
+		DBG_DEC(tFootnoteLen);
 		werr(1, "Software error (Footnotes)");
 	}
-	iTotal = 0;
+	tTotal = 0;
 	for (pCurr = pUnused1Anchor; pCurr != NULL; pCurr = pCurr->pNext) {
-		DBG_HEX(pCurr->tInfo.iFileOffset);
-		NO_DBG_HEX(pCurr->tInfo.iTextOffset);
-		DBG_DEC(pCurr->tInfo.iLength);
-		fail(pCurr->tInfo.iLength <= 0);
+		DBG_HEX(pCurr->tInfo.lFileOffset);
+		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
+		DBG_DEC(pCurr->tInfo.tLength);
+		fail(pCurr->tInfo.tLength == 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.iLength));
-			iTotal += pCurr->tInfo.iLength / 2;
+			fail(odd(pCurr->tInfo.tLength));
+			tTotal += pCurr->tInfo.tLength / 2;
 		} else {
-			iTotal += pCurr->tInfo.iLength;
+			tTotal += pCurr->tInfo.tLength;
 		}
 	}
-	DBG_DEC(iTotal);
-	if (iTotal != iUnused1Len) {
-		DBG_DEC(iUnused1Len);
+	DBG_DEC(tTotal);
+	if (tTotal != tUnused1Len) {
+		DBG_DEC(tUnused1Len);
 		werr(1, "Software error (Unused1-list)");
 	}
-	iTotal = 0;
+	tTotal = 0;
 	for (pCurr = pEndAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
-		DBG_HEX(pCurr->tInfo.iFileOffset);
-		NO_DBG_HEX(pCurr->tInfo.iTextOffset);
-		DBG_DEC(pCurr->tInfo.iLength);
-		fail(pCurr->tInfo.iLength <= 0);
+		DBG_HEX(pCurr->tInfo.lFileOffset);
+		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
+		DBG_DEC(pCurr->tInfo.tLength);
+		fail(pCurr->tInfo.tLength == 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.iLength));
-			iTotal += pCurr->tInfo.iLength / 2;
+			fail(odd(pCurr->tInfo.tLength));
+			tTotal += pCurr->tInfo.tLength / 2;
 		} else {
-			iTotal += pCurr->tInfo.iLength;
+			tTotal += pCurr->tInfo.tLength;
 		}
 	}
-	DBG_DEC(iTotal);
-	if (iTotal != iEndnoteLen) {
-		DBG_DEC(iEndnoteLen);
+	DBG_DEC(tTotal);
+	if (tTotal != tEndnoteLen) {
+		DBG_DEC(tEndnoteLen);
 		werr(1, "Software error (Endnotes)");
 	}
-	iTotal = 0;
+	tTotal = 0;
 	for (pCurr = pUnused2Anchor; pCurr != NULL; pCurr = pCurr->pNext) {
-		DBG_HEX(pCurr->tInfo.iFileOffset);
-		NO_DBG_HEX(pCurr->tInfo.iTextOffset);
-		DBG_DEC(pCurr->tInfo.iLength);
-		fail(pCurr->tInfo.iLength <= 0);
+		DBG_HEX(pCurr->tInfo.lFileOffset);
+		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
+		DBG_DEC(pCurr->tInfo.tLength);
+		fail(pCurr->tInfo.tLength == 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.iLength));
-			iTotal += pCurr->tInfo.iLength / 2;
+			fail(odd(pCurr->tInfo.tLength));
+			tTotal += pCurr->tInfo.tLength / 2;
 		} else {
-			iTotal += pCurr->tInfo.iLength;
+			tTotal += pCurr->tInfo.tLength;
 		}
 	}
-	DBG_DEC(iTotal);
-	if (iTotal != iUnused2Len) {
-		DBG_DEC(iUnused2Len);
+	DBG_DEC(tTotal);
+	if (tTotal != tUnused2Len) {
+		DBG_DEC(tUnused2Len);
 		werr(1, "Software error (Unused2-list)");
 	}
 #endif /* DEBUG */
@@ -585,30 +587,32 @@ vSplitBlockList(int iTextLen, int iFootnoteLen,
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
 			if (pCurr->pNext != NULL &&
-			    pCurr->tInfo.iLength % BIG_BLOCK_SIZE != 0) {
-				DBG_HEX(pCurr->tInfo.iFileOffset);
-				DBG_HEX(pCurr->tInfo.iTextOffset);
-				DBG_DEC(pCurr->tInfo.iLength);
-				pCurr->tInfo.iLength /= BIG_BLOCK_SIZE;
-				pCurr->tInfo.iLength++;
-				pCurr->tInfo.iLength *= BIG_BLOCK_SIZE;
-				DBG_DEC(pCurr->tInfo.iLength);
+			    pCurr->tInfo.tLength % BIG_BLOCK_SIZE != 0) {
+				DBG_HEX(pCurr->tInfo.lFileOffset);
+				DBG_HEX(pCurr->tInfo.lTextOffset);
+				DBG_DEC(pCurr->tInfo.tLength);
+				pCurr->tInfo.tLength /= BIG_BLOCK_SIZE;
+				pCurr->tInfo.tLength++;
+				pCurr->tInfo.tLength *= BIG_BLOCK_SIZE;
+				DBG_DEC(pCurr->tInfo.tLength);
 			}
 		}
 	}
 } /* end of vSplitBlockList */
 
+#if defined(__riscos)
 /*
- * uiGetDocumentLength - get the combined character length of the three lists
+ * tGetDocumentLength - get the combined character length of the three lists
  *
  * returns: The total number of characters
  */
-unsigned int
-uiGetDocumentLength(void)
+size_t
+tGetDocumentLength(void)
 {
 	list_mem_type	*apAnchors[3];
 	list_mem_type	*pCurr;
-	int	iTotal, iIndex;
+	size_t		tTotal;
+	int		iIndex;
 
 	DBG_MSG("uiGetDocumentLength");
 
@@ -616,271 +620,272 @@ uiGetDocumentLength(void)
 	apAnchors[1] = pFootAnchor;
 	apAnchors[2] = pEndAnchor;
 
-	iTotal = 0;
+	tTotal = 0;
 	for (iIndex = 0; iIndex < 3; iIndex++) {
 		for (pCurr = apAnchors[iIndex];
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			fail(pCurr->tInfo.iLength <= 0);
+			fail(pCurr->tInfo.tLength == 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.iLength));
-				iTotal += pCurr->tInfo.iLength / 2;
+				fail(odd(pCurr->tInfo.tLength));
+				tTotal += pCurr->tInfo.tLength / 2;
 			} else {
-				iTotal += pCurr->tInfo.iLength;
+				tTotal += pCurr->tInfo.tLength;
 			}
 		}
 	}
-	DBG_DEC(iTotal);
-	return iTotal;
-} /* end of uiGetDocumentLength */
+	DBG_DEC(tTotal);
+	return tTotal;
+} /* end of tGetDocumentLength */
+#endif /* __riscos */
 
 /*
- * iNextTextByte - get the next byte from the text block list
+ * usNextTextByte - get the next byte from the text block list
  */
-static int
-iNextTextByte(FILE *pFile, int *piOffset)
+static unsigned short
+usNextTextByte(FILE *pFile, long *plOffset)
 {
-	static int	iByteNext = 0;
-	static int	iBlockOffset = 0;
-	int	iReadLen, iReadOff;
+	static size_t	tByteNext = 0;
+	static size_t	tBlockOffset = 0;
+	long	lReadOff;
+	size_t	tReadLen;
 
 	if (pTextBlockCurrent == NULL ||
-	    iByteNext >= sizeof(aucBlock) ||
-	    iBlockOffset + iByteNext >= pTextBlockCurrent->tInfo.iLength) {
+	    tByteNext >= sizeof(aucBlock) ||
+	    tBlockOffset + tByteNext >= pTextBlockCurrent->tInfo.tLength) {
 		if (pTextBlockCurrent == NULL) {
 			/* First block, first part */
 			pTextBlockCurrent = pTextAnchor;
-			iBlockOffset = 0;
-		} else if (iBlockOffset + sizeof(aucBlock) <
-					pTextBlockCurrent->tInfo.iLength) {
+			tBlockOffset = 0;
+		} else if (tBlockOffset + sizeof(aucBlock) <
+				pTextBlockCurrent->tInfo.tLength) {
 			/* Same block, next part */
-			iBlockOffset += sizeof(aucBlock);
+			tBlockOffset += sizeof(aucBlock);
 		} else {
 			/* Next block, first part */
 			pTextBlockCurrent = pTextBlockCurrent->pNext;
-			iBlockOffset = 0;
+			tBlockOffset = 0;
 		}
 		if (pTextBlockCurrent == NULL) {
 			/* Past the last part of the last block */
-			return EOF;
+			return (unsigned short)EOF;
 		}
-		iReadLen = pTextBlockCurrent->tInfo.iLength - iBlockOffset;
-		if (iReadLen > sizeof(aucBlock)) {
-			iReadLen = sizeof(aucBlock);
+		tReadLen = pTextBlockCurrent->tInfo.tLength - tBlockOffset;
+		if (tReadLen > sizeof(aucBlock)) {
+			tReadLen = sizeof(aucBlock);
 		}
-		iReadOff = pTextBlockCurrent->tInfo.iFileOffset +
-				iBlockOffset;
-		if (!bReadBytes(aucBlock, iReadLen, iReadOff, pFile)) {
-			return EOF;
+		lReadOff = pTextBlockCurrent->tInfo.lFileOffset +
+				(long)tBlockOffset;
+		if (!bReadBytes(aucBlock, tReadLen, lReadOff, pFile)) {
+			return (unsigned short)EOF;
 		}
-		iByteNext = 0;
+		tByteNext = 0;
 	}
-	if (piOffset != NULL) {
-		*piOffset = pTextBlockCurrent->tInfo.iFileOffset +
-			iBlockOffset +
-			iByteNext;
+	if (plOffset != NULL) {
+		*plOffset = pTextBlockCurrent->tInfo.lFileOffset +
+			(long)tBlockOffset +
+			(long)tByteNext;
 	}
-	return aucBlock[iByteNext++];
-} /* end of iNextTextByte */
+	return aucBlock[tByteNext++];
+} /* end of usNextTextByte */
 
 /*
- * iNextFootByte - get the next byte from the footnote block list
+ * usNextFootByte - get the next byte from the footnote block list
  */
-static int
-iNextFootByte(FILE *pFile, int *piOffset)
+static unsigned short
+usNextFootByte(FILE *pFile, long *plOffset)
 {
-	static int	iByteNext = 0;
-	static int	iBlockOffset = 0;
-	int	iReadLen, iReadOff;
+	static size_t	tByteNext = 0;
+	static size_t	tBlockOffset = 0;
+	long	lReadOff;
+	size_t	tReadLen;
 
 	if (pFootBlockCurrent == NULL ||
-	    iByteNext >= sizeof(aucBlock) ||
-	    iBlockOffset + iByteNext >= pFootBlockCurrent->tInfo.iLength) {
+	    tByteNext >= sizeof(aucBlock) ||
+	    tBlockOffset + tByteNext >= pFootBlockCurrent->tInfo.tLength) {
 		if (pFootBlockCurrent == NULL) {
 			/* First block, first part */
 			pFootBlockCurrent = pFootAnchor;
-			iBlockOffset = 0;
-		} else if (iBlockOffset + sizeof(aucBlock) <
-					pFootBlockCurrent->tInfo.iLength) {
+			tBlockOffset = 0;
+		} else if (tBlockOffset + sizeof(aucBlock) <
+				pFootBlockCurrent->tInfo.tLength) {
 			/* Same block, next part */
-			iBlockOffset += sizeof(aucBlock);
+			tBlockOffset += sizeof(aucBlock);
 		} else {
 			/* Next block, first part */
 			pFootBlockCurrent = pFootBlockCurrent->pNext;
-			iBlockOffset = 0;
+			tBlockOffset = 0;
 		}
 		if (pFootBlockCurrent == NULL) {
 			/* Past the last part of the last block */
-			return EOF;
+			return (unsigned short)EOF;
 		}
-		iReadLen = pFootBlockCurrent->tInfo.iLength - iBlockOffset;
-		if (iReadLen > sizeof(aucBlock)) {
-			iReadLen = sizeof(aucBlock);
+		tReadLen = pFootBlockCurrent->tInfo.tLength - tBlockOffset;
+		if (tReadLen > sizeof(aucBlock)) {
+			tReadLen = sizeof(aucBlock);
 		}
-		iReadOff = pFootBlockCurrent->tInfo.iFileOffset +
-				iBlockOffset;
-		if (!bReadBytes(aucBlock, iReadLen, iReadOff, pFile)) {
-			return EOF;
+		lReadOff = pFootBlockCurrent->tInfo.lFileOffset +
+				(long)tBlockOffset;
+		if (!bReadBytes(aucBlock, tReadLen, lReadOff, pFile)) {
+			return (unsigned short)EOF;
 		}
-		iByteNext = 0;
+		tByteNext = 0;
 	}
-	if (piOffset != NULL) {
-		*piOffset = pFootBlockCurrent->tInfo.iFileOffset +
-			iBlockOffset +
-			iByteNext;
+	if (plOffset != NULL) {
+		*plOffset = pFootBlockCurrent->tInfo.lFileOffset +
+			(long)tBlockOffset +
+			(long)tByteNext;
 	}
-	return aucBlock[iByteNext++];
-} /* end of iNextFootByte */
+	return aucBlock[tByteNext++];
+} /* end of usNextFootByte */
 
 /*
- * iNextEndByte - get the next byte from the endnote block list
+ * usNextEndByte - get the next byte from the endnote block list
  */
-static int
-iNextEndByte(FILE *pFile, int *piOffset)
+static unsigned short
+usNextEndByte(FILE *pFile, long *plOffset)
 {
-	static int	iByteNext = 0;
-	static int	iBlockOffset = 0;
-	int	iReadLen, iReadOff;
+	static size_t	tByteNext = 0;
+	static size_t	tBlockOffset = 0;
+	long	lReadOff;
+	size_t	tReadLen;
 
 	if (pEndBlockCurrent == NULL ||
-	    iByteNext >= sizeof(aucBlock) ||
-	    iBlockOffset + iByteNext >= pEndBlockCurrent->tInfo.iLength) {
+	    tByteNext >= sizeof(aucBlock) ||
+	    tBlockOffset + tByteNext >= pEndBlockCurrent->tInfo.tLength) {
 		if (pEndBlockCurrent == NULL) {
 			/* First block, first part */
 			pEndBlockCurrent = pEndAnchor;
-			iBlockOffset = 0;
-		} else if (iBlockOffset + sizeof(aucBlock) <
-					pEndBlockCurrent->tInfo.iLength) {
+			tBlockOffset = 0;
+		} else if (tBlockOffset + sizeof(aucBlock) <
+				pEndBlockCurrent->tInfo.tLength) {
 			/* Same block, next part */
-			iBlockOffset += sizeof(aucBlock);
+			tBlockOffset += sizeof(aucBlock);
 		} else {
 			/* Next block, first part */
 			pEndBlockCurrent = pEndBlockCurrent->pNext;
-			iBlockOffset = 0;
+			tBlockOffset = 0;
 		}
 		if (pEndBlockCurrent == NULL) {
 			/* Past the last part of the last block */
-			return EOF;
+			return (unsigned short)EOF;
 		}
-		iReadLen = pEndBlockCurrent->tInfo.iLength - iBlockOffset;
-		if (iReadLen > sizeof(aucBlock)) {
-			iReadLen = sizeof(aucBlock);
+		tReadLen = pEndBlockCurrent->tInfo.tLength - tBlockOffset;
+		if (tReadLen > sizeof(aucBlock)) {
+			tReadLen = sizeof(aucBlock);
 		}
-		iReadOff = pEndBlockCurrent->tInfo.iFileOffset +
-				iBlockOffset;
-		if (!bReadBytes(aucBlock, iReadLen, iReadOff, pFile)) {
-			return EOF;
+		lReadOff = pEndBlockCurrent->tInfo.lFileOffset +
+				(long)tBlockOffset;
+		if (!bReadBytes(aucBlock, tReadLen, lReadOff, pFile)) {
+			return (unsigned short)EOF;
 		}
-		iByteNext = 0;
+		tByteNext = 0;
 	}
-	if (piOffset != NULL) {
-		*piOffset = pEndBlockCurrent->tInfo.iFileOffset +
-			iBlockOffset +
-			iByteNext;
+	if (plOffset != NULL) {
+		*plOffset = pEndBlockCurrent->tInfo.lFileOffset +
+			(long)tBlockOffset +
+			(long)tByteNext;
 	}
-	return aucBlock[iByteNext++];
-} /* end of iNextEndByte */
+	return aucBlock[tByteNext++];
+} /* end of usNextEndByte */
 
 /*
- * iNextTextChar - get the next character from the text block list
+ * usNextTextChar - get the next character from the text block list
  */
-static int
-iNextTextChar(FILE *pFile, int *piOffset)
+static unsigned short
+usNextTextChar(FILE *pFile, long *plOffset)
 {
-	int	iChar, iMSB;
+	unsigned short	usLSB, usMSB;
 
-	iChar = iNextTextByte(pFile, piOffset);
-	if (iChar == EOF) {
-		return EOF;
+	usLSB = usNextTextByte(pFile, plOffset);
+	if (usLSB == (unsigned short)EOF) {
+		return (unsigned short)EOF;
 	}
 	if (pTextBlockCurrent->tInfo.bUsesUnicode) {
-		iMSB = iNextTextByte(pFile, NULL);
+		usMSB = usNextTextByte(pFile, NULL);
 	} else {
-		iMSB = 0x00;
+		usMSB = 0x00;
 	}
-	if (iMSB == EOF) {
-		DBG_MSG("iNextTextChar: Unexpected EOF");
-		DBG_HEX_C(piOffset != NULL, *piOffset);
-		return EOF;
+	if (usMSB == (unsigned short)EOF) {
+		DBG_MSG("usNextTextChar: Unexpected EOF");
+		DBG_HEX_C(plOffset != NULL, *plOffset);
+		return (unsigned short)EOF;
 	}
-	iChar |= iMSB << 8;
-	return iChar;
-} /* end of iNextTextChar */
+	return (usMSB << 8) | usLSB;
+} /* end of usNextTextChar */
 
 /*
- * iNextFootChar - get the next character from the footnote block list
+ * usNextFootChar - get the next character from the footnote block list
  */
-static int
-iNextFootChar(FILE *pFile, int *piOffset)
+static unsigned short
+usNextFootChar(FILE *pFile, long *plOffset)
 {
-	int	iChar, iMSB;
+	unsigned short	usLSB, usMSB;
 
-	iChar = iNextFootByte(pFile, piOffset);
-	if (iChar == EOF) {
-		return EOF;
+	usLSB = usNextFootByte(pFile, plOffset);
+	if (usLSB == (unsigned short)EOF) {
+		return (unsigned short)EOF;
 	}
 	if (pFootBlockCurrent->tInfo.bUsesUnicode) {
-		iMSB = iNextFootByte(pFile, NULL);
+		usMSB = usNextFootByte(pFile, NULL);
 	} else {
-		iMSB = 0x00;
+		usMSB = 0x00;
 	}
-	if (iMSB == EOF) {
-		DBG_MSG("iNextFootChar: Unexpected EOF");
-		DBG_HEX_C(piOffset != NULL, *piOffset);
-		return EOF;
+	if (usMSB == (unsigned short)EOF) {
+		DBG_MSG("usNextFootChar: Unexpected EOF");
+		DBG_HEX_C(plOffset != NULL, *plOffset);
+		return (unsigned short)EOF;
 	}
-	iChar |= iMSB << 8;
-	return iChar;
-} /* end of iNextFootChar */
+	return (usMSB << 8) | usLSB;
+} /* end of usNextFootChar */
 
 /*
- * iNextEndChar - get the next character from the endnote block list
+ * usNextEndChar - get the next character from the endnote block list
  */
-static int
-iNextEndChar(FILE *pFile, int *piOffset)
+static unsigned short
+usNextEndChar(FILE *pFile, long *plOffset)
 {
-	int	iChar, iMSB;
+	unsigned short	usLSB, usMSB;
 
-	iChar = iNextEndByte(pFile, piOffset);
-	if (iChar == EOF) {
-		return EOF;
+	usLSB = usNextEndByte(pFile, plOffset);
+	if (usLSB == (unsigned short)EOF) {
+		return (unsigned short)EOF;
 	}
 	if (pEndBlockCurrent->tInfo.bUsesUnicode) {
-		iMSB = iNextEndByte(pFile, NULL);
+		usMSB = usNextEndByte(pFile, NULL);
 	} else {
-		iMSB = 0x00;
+		usMSB = 0x00;
 	}
-	if (iMSB == EOF) {
-		DBG_MSG("iNextEndChar: Unexpected EOF");
-		DBG_HEX_C(piOffset != NULL, *piOffset);
-		return EOF;
+	if (usMSB == (unsigned short)EOF) {
+		DBG_MSG("usNextEndChar: Unexpected EOF");
+		DBG_HEX_C(plOffset != NULL, *plOffset);
+		return (unsigned short)EOF;
 	}
-	iChar |= iMSB << 8;
-	return iChar;
-} /* end of iNextEndChar */
+	return (usMSB << 8) | usLSB;
+} /* end of usNextEndChar */
 
 /*
- * iNextChar - get the next character from the given block list
+ * usNextChar - get the next character from the given block list
  */
-int
-iNextChar(FILE *pFile, list_id_enum eListID, int *piOffset)
+unsigned short
+usNextChar(FILE *pFile, list_id_enum eListID, long *plOffset)
 {
 	fail(pFile == NULL);
 
 	switch (eListID) {
 	case text_list:
-		return iNextTextChar(pFile, piOffset);
+		return usNextTextChar(pFile, plOffset);
 	case footnote_list:
-		return iNextFootChar(pFile, piOffset);
+		return usNextFootChar(pFile, plOffset);
 	case endnote_list:
-		return iNextEndChar(pFile, piOffset);
+		return usNextEndChar(pFile, plOffset);
 	default:
-		if (piOffset != NULL) {
-			*piOffset = -1;
+		if (plOffset != NULL) {
+			*plOffset = -1;
 		}
-		return EOF;
+		return (unsigned short)EOF;
 	}
-} /* end of iNextChar */
+} /* end of usNextChar */
 
 /*
  * Translate the start from the begin of the text to an offset in the file.
@@ -888,20 +893,14 @@ iNextChar(FILE *pFile, list_id_enum eListID, int *piOffset)
  * Returns:	 -1: in case of error
  *		>=0: the computed file offset
  */
-int
-iTextOffset2FileOffset(int iTextOffset)
+long
+lTextOffset2FileOffset(long lTextOffset)
 {
 	list_mem_type	*apAnchors[5];
 	list_mem_type	*pCurr;
 	int		iIndex;
 	BOOL		bStartOfList;
 
-	fail(iTextOffset < 0);
-
-	if (iTextOffset < 0) {
-		return -1;
-	}
-
 	apAnchors[0] = pTextAnchor;
 	apAnchors[1] = pFootAnchor;
 	apAnchors[2] = pUnused1Anchor;
@@ -915,16 +914,16 @@ iTextOffset2FileOffset(int iTextOffset)
 		while (pCurr != NULL) {
 			if (bStartOfList) {
 				NO_DBG_DEC(iIndex);
-				NO_DBG_HEX(pCurr->tInfo.iFileOffset);
+				NO_DBG_HEX(pCurr->tInfo.lFileOffset);
 				if (iIndex >= 4) {
 					/* Past the last used byte */
 					return -1;
 				}
-				return pCurr->tInfo.iFileOffset;
+				return pCurr->tInfo.lFileOffset;
 			}
-			if (iTextOffset < pCurr->tInfo.iTextOffset ||
-			    iTextOffset >= pCurr->tInfo.iTextOffset +
-			     pCurr->tInfo.iLength) {
+			if (lTextOffset < pCurr->tInfo.lTextOffset ||
+			    lTextOffset >= pCurr->tInfo.lTextOffset +
+			     (long)pCurr->tInfo.tLength) {
 				pCurr = pCurr->pNext;
 				continue;
 			}
@@ -933,9 +932,9 @@ iTextOffset2FileOffset(int iTextOffset)
 			case 1:
 			case 3:
 			/* The textoffset is in the current block */
-				return pCurr->tInfo.iFileOffset +
-					iTextOffset -
-					pCurr->tInfo.iTextOffset;
+				return pCurr->tInfo.lFileOffset +
+					lTextOffset -
+					pCurr->tInfo.lTextOffset;
 			case 2:
 			/* Use the start of the next non-empty list */
 				bStartOfList = TRUE;
@@ -947,13 +946,39 @@ iTextOffset2FileOffset(int iTextOffset)
 			/* This should not happen */
 				return -1;
 			}
-			if (bStartOfList) {
-				/* To the start of the next list */
-				break;
-			}
+			/* To the start of the next list */
+			break;
 		}
 	}
 	/* Passed beyond the end of the last list */
-	DBG_HEX(iTextOffset);
+	NO_DBG_HEX(lTextOffset);
 	return -1;
-} /* end of iTextOffset2FileOffset */
+} /* end of lTextOffset2FileOffset */
+
+/*
+ * Get the sequence number beloning to the given file offset
+ *
+ * Returns the sequence number
+ */
+long
+lGetSeqNumber(long lFileOffset)
+{
+	list_mem_type	*pCurr;
+	long		lSeq;
+
+	if (lFileOffset < 0) {
+		return -1;
+	}
+
+	lSeq = 0;
+	for (pCurr = pTextAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
+		if (lFileOffset >= pCurr->tInfo.lFileOffset &&
+		    lFileOffset < pCurr->tInfo.lFileOffset +
+		     (long)pCurr->tInfo.tLength) {
+			/* The file offset is within the current textblock */
+			return lSeq + lFileOffset - pCurr->tInfo.lFileOffset;
+		}
+		lSeq += (long)pCurr->tInfo.tLength;
+	}
+	return -1;
+} /* end of lGetSeqNumber */
