@@ -1,6 +1,6 @@
 /*
  * word2text.c
- * Copyright (C) 1998,1999 A.J. van Os
+ * Copyright (C) 1998-2000 A.J. van Os; Released under GPL
  *
  * Description:
  * MS Word to text functions
@@ -10,7 +10,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <limits.h>
 #if defined(__riscos)
 #include "visdelay.h"
 #endif /* __riscos */
@@ -24,7 +23,7 @@
 /* Macros to make sure all such statements will be identical */
 #define OUTPUT_LINE()		\
 	do {\
-		vAlign2Window(pDiag, pAnchor, iWidthMax, ucAlignment);\
+		vAlign2Window(pDiag, pAnchor, lWidthMax, ucAlignment);\
 		pAnchor = pStartNewOutput(pAnchor, NULL);\
 		pOutput = pAnchor;\
 	} while(0)
@@ -35,16 +34,18 @@
 		pOutput = pAnchor;\
 	} while(0)
 
+#if defined(__riscos)
 /* Length of the document in characters */
-static unsigned int	uiDocumentLength;
+static size_t	tDocumentLength;
 /* Number of characters processed so far */
-static unsigned int	uiCharCounter;
-static int		iCurrPct, iPrevPct;
+static size_t	tCharCounter;
+static int	iCurrPct, iPrevPct;
+#endif /* __riscos */
 /* Special treatment for files from the Apple Macintosh */
-static BOOL		bMacFile = FALSE;
+static BOOL	bMacFile = FALSE;
 /* Needed for reading a complete table row */
 static row_block_type	tRowInfo;
-static BOOL		bRowInfo = FALSE;
+static BOOL	bRowInfo = FALSE;
 static BOOL	bStartRow = FALSE;
 static BOOL	bEndRow = FALSE;
 static BOOL	bIsTableRow = FALSE;
@@ -54,6 +55,8 @@ static BOOL	bStartStyle = FALSE;
 /* Needed for finding the start of a font */
 static const font_block_type	*pFontInfo = NULL;
 static BOOL	bStartFont = FALSE;
+/* Needed for finding an image */
+static long	lFileOffsetImage = -1;
 
 
 /*
@@ -63,8 +66,8 @@ static void
 vUpdateCounters(void)
 {
 #if defined(__riscos)
-	uiCharCounter++;
-	iCurrPct = (int)((uiCharCounter * 100) / uiDocumentLength);
+	tCharCounter++;
+	iCurrPct = (int)((tCharCounter * 100) / tDocumentLength);
 	if (iCurrPct != iPrevPct) {
 		visdelay_percent(iCurrPct);
 		iPrevPct = iCurrPct;
@@ -84,7 +87,7 @@ bOutputContainsText(output_type *pAnchor)
 	fail(pAnchor == NULL);
 
 	for (pTmp = pAnchor; pTmp != NULL; pTmp = pTmp->pNext) {
-		fail(pTmp->iStringWidth < 0);
+		fail(pTmp->lStringWidth < 0);
 		for (iIndex = 0; iIndex < pTmp->iNextFree; iIndex++) {
 			if (isspace(pTmp->szStorage[iIndex])) {
 				continue;
@@ -102,21 +105,21 @@ bOutputContainsText(output_type *pAnchor)
 } /* end of bOutputContainsText */
 
 /*
- * iTotalStringWidth - compute the total width of the output string
+ * lTotalStringWidth - compute the total width of the output string
  */
-static int
-iTotalStringWidth(output_type *pAnchor)
+static long
+lTotalStringWidth(output_type *pAnchor)
 {
 	output_type	*pTmp;
-	int		iTotal;
+	long		lTotal;
 
-	iTotal = 0;
+	lTotal = 0;
 	for (pTmp = pAnchor; pTmp != NULL; pTmp = pTmp->pNext) {
-		fail(pTmp->iStringWidth < 0);
-		iTotal += pTmp->iStringWidth;
+		fail(pTmp->lStringWidth < 0);
+		lTotal += pTmp->lStringWidth;
 	}
-	return iTotal;
-} /* end of iTotalStringWidth */
+	return lTotal;
+} /* end of lTotalStringWidth */
 
 /*
  * vStoreCharacter - store one character
@@ -130,18 +133,18 @@ vStoreCharacter(int iChar, output_type *pOutput)
 		pOutput->szStorage[pOutput->iNextFree] = '\0';
 		return;
 	}
-	while (pOutput->iNextFree + 2 > pOutput->iStorageSize) {
-		pOutput->iStorageSize += EXTENTION_SIZE;
+	while (pOutput->iNextFree + 2 > (int)pOutput->tStorageSize) {
+		pOutput->tStorageSize += EXTENTION_SIZE;
 		pOutput->szStorage = xrealloc(pOutput->szStorage,
-					pOutput->iStorageSize);
+					pOutput->tStorageSize);
 	}
 	pOutput->szStorage[pOutput->iNextFree] = (char)iChar;
 	pOutput->szStorage[pOutput->iNextFree+1] = '\0';
-	pOutput->iStringWidth += iComputeStringWidth(
+	pOutput->lStringWidth += lComputeStringWidth(
 				pOutput->szStorage + pOutput->iNextFree,
 				1,
 				pOutput->tFontRef,
-				pOutput->ucFontsize);
+				pOutput->sFontsize);
 	pOutput->iNextFree++;
 } /* end of vStoreCharacter */
 
@@ -195,7 +198,7 @@ vStoreIntegerAsRoman(int iNumber, output_type *pOutput)
  * vStoreStyle - store a style
  */
 static void
-vStoreStyle(const style_block_type *pStyleInfo, output_type *pOutput)
+vStoreStyle(output_type *pOutput)
 {
 	int	iLen;
 	char	szString[120];
@@ -205,6 +208,23 @@ vStoreStyle(const style_block_type *pStyleInfo, output_type *pOutput)
 	vStoreString(szString, iLen, pOutput);
 } /* end of vStoreStyle */
 
+/*
+ * Create an empty line by adding a extra "newline"
+ */
+static void
+vEmptyLine2Diagram(diagram_type *pDiag, draw_fontref tFontRef, int iFontsize)
+{
+	fail(pDiag == NULL);
+	fail(iFontsize < MIN_FONT_SIZE || iFontsize > MAX_FONT_SIZE);
+
+	if (pDiag->lXleft > 0) {
+		/* To the start of the line */
+		vMove2NextLine(pDiag, tFontRef, iFontsize);
+	}
+	/* Empty line */
+	vMove2NextLine(pDiag, tFontRef, iFontsize);
+} /* end of vEmptyLine2Diagram */
+
 /*
  * vPutIndentation - output the given amount of indentation
  */
@@ -213,7 +233,8 @@ vPutIndentation(diagram_type *pDiag, output_type *pOutput, BOOL bUnmarked,
 	int iListNumber, unsigned char ucListType, char cListCharacter,
 	int iLeftIndent)
 {
-	int	iLeftIndentation, iWidth, iNextFree;
+	long	lWidth, lLeftIndentation;
+	int	iNextFree;
 	char	szLine[30];
 
 	fail(pDiag == NULL || pOutput == NULL);
@@ -223,9 +244,9 @@ vPutIndentation(diagram_type *pDiag, output_type *pOutput, BOOL bUnmarked,
 	if (iLeftIndent <= 0) {
 		return;
 	}
-	iLeftIndentation = iTwips2MilliPoints(iLeftIndent);
+	lLeftIndentation = lTwips2MilliPoints(iLeftIndent);
 	if (bUnmarked) {
-		vSetLeftIndentation(pDiag, iLeftIndentation);
+		vSetLeftIndentation(pDiag, lLeftIndentation);
 		return;
 	}
 	fail(iListNumber <= 0 || iscntrl(cListCharacter));
@@ -251,11 +272,11 @@ vPutIndentation(diagram_type *pDiag, output_type *pOutput, BOOL bUnmarked,
 	szLine[iNextFree++] = cListCharacter;
 	szLine[iNextFree++] = ' ';
 	szLine[iNextFree] = '\0';
-	iWidth = iComputeStringWidth(szLine, iNextFree,
-				pOutput->tFontRef, pOutput->ucFontsize);
-	iLeftIndentation -= iWidth;
-	if (iLeftIndentation > 0) {
-		vSetLeftIndentation(pDiag, iLeftIndentation);
+	lWidth = lComputeStringWidth(szLine, iNextFree,
+				pOutput->tFontRef, pOutput->sFontsize);
+	lLeftIndentation -= lWidth;
+	if (lLeftIndentation > 0) {
+		vSetLeftIndentation(pDiag, lLeftIndentation);
 	}
 	vStoreString(szLine, iNextFree, pOutput);
 } /* end of vPutIndentation */
@@ -269,17 +290,18 @@ vPutIndentation(diagram_type *pDiag, output_type *pOutput, BOOL bUnmarked,
 static void
 vPutNoteSeparator(output_type *pOutput)
 {
-	int	iCounter, iChars, iCharWidth;
+	long	lCharWidth;
+	int	iCounter, iChars;
 	char	szOne[2];
 
 	fail(pOutput == NULL);
 
 	szOne[0] = OUR_EM_DASH;
 	szOne[1] = '\0';
-	iCharWidth = iComputeStringWidth(szOne, 1,
-				pOutput->tFontRef, pOutput->ucFontsize);
-	DBG_DEC(iCharWidth);
-	iChars = (144000 + iCharWidth / 2) / iCharWidth;
+	lCharWidth = lComputeStringWidth(szOne, 1,
+				pOutput->tFontRef, pOutput->sFontsize);
+	DBG_DEC(lCharWidth);
+	iChars = (int)((144000 + lCharWidth / 2) / lCharWidth);
 	DBG_DEC(iChars);
 	for (iCounter = 0; iCounter < iChars; iCounter++) {
 		vStoreCharacter(OUR_EM_DASH, pOutput);
@@ -297,21 +319,21 @@ pStartNextOutput(output_type *pCurrent)
 	if (pCurrent->iNextFree == 0) {
 		/* The current record is empty, re-use */
 		fail(pCurrent->szStorage[0] != '\0');
-		fail(pCurrent->iStringWidth != 0);
+		fail(pCurrent->lStringWidth != 0);
 		return pCurrent;
 	}
 	/* The current record is in use, make a new one */
 	pNew = xmalloc(sizeof(*pNew));
 	pCurrent->pNext = pNew;
-	pNew->iStorageSize = INITIAL_SIZE;
-	pNew->szStorage = xmalloc(pNew->iStorageSize);
+	pNew->tStorageSize = INITIAL_SIZE;
+	pNew->szStorage = xmalloc(pNew->tStorageSize);
 	pNew->szStorage[0] = '\0';
 	pNew->iNextFree = 0;
-	pNew->iStringWidth = 0;
-	pNew->iColour = 0;
+	pNew->lStringWidth = 0;
+	pNew->iColor = FONT_COLOR_DEFAULT;
 	pNew->ucFontstyle = FONT_REGULAR;
 	pNew->tFontRef = 0;
-	pNew->ucFontsize = DEFAULT_FONT_SIZE;
+	pNew->sFontsize = DEFAULT_FONT_SIZE;
 	pNew->pPrev = pCurrent;
 	pNew->pNext = NULL;
 	return pNew;
@@ -324,24 +346,25 @@ static output_type *
 pStartNewOutput(output_type *pAnchor, output_type *pLeftOver)
 {
 	output_type	*pCurr, *pNext;
-	int		iColour;
+	int		iColor;
+	short		sFontsize;
 	draw_fontref	tFontRef;
-	unsigned char	ucFontstyle, ucFontsize;
+	unsigned char	ucFontstyle;
 
-	iColour = FONT_COLOUR_DEFAULT;
+	iColor = FONT_COLOR_DEFAULT;
 	ucFontstyle = FONT_REGULAR;
 	tFontRef = 0;
-	ucFontsize = DEFAULT_FONT_SIZE;
+	sFontsize = DEFAULT_FONT_SIZE;
 	/* Free the old output space */
 	pCurr = pAnchor;
 	while (pCurr != NULL) {
 		pNext = pCurr->pNext;
 		pCurr->szStorage = xfree(pCurr->szStorage);
 		if (pCurr->pNext == NULL) {
-			iColour = pCurr->iColour;
+			iColor = pCurr->iColor;
 			ucFontstyle = pCurr->ucFontstyle;
 			tFontRef = pCurr->tFontRef;
-			ucFontsize = pCurr->ucFontsize;
+			sFontsize = pCurr->sFontsize;
 		}
 		pCurr = xfree(pCurr);
 		pCurr = pNext;
@@ -349,15 +372,15 @@ pStartNewOutput(output_type *pAnchor, output_type *pLeftOver)
 	if (pLeftOver == NULL) {
 		/* Create new output space */
 		pLeftOver = xmalloc(sizeof(*pLeftOver));
-		pLeftOver->iStorageSize = INITIAL_SIZE;
-		pLeftOver->szStorage = xmalloc(pLeftOver->iStorageSize);
+		pLeftOver->tStorageSize = INITIAL_SIZE;
+		pLeftOver->szStorage = xmalloc(pLeftOver->tStorageSize);
 		pLeftOver->szStorage[0] = '\0';
 		pLeftOver->iNextFree = 0;
-		pLeftOver->iStringWidth = 0;
-		pLeftOver->iColour = iColour;
+		pLeftOver->lStringWidth = 0;
+		pLeftOver->iColor = iColor;
 		pLeftOver->ucFontstyle = ucFontstyle;
 		pLeftOver->tFontRef = tFontRef;
-		pLeftOver->ucFontsize = ucFontsize;
+		pLeftOver->sFontsize = sFontsize;
 		pLeftOver->pPrev = NULL;
 		pLeftOver->pNext = NULL;
 	}
@@ -372,7 +395,9 @@ static int
 iGetChar(FILE *pFile, list_id_enum eListID)
 {
 	const font_block_type *pCurr;
-	int	iChar, iFileOffset;
+	long	lFileOffset;
+	int	iChar;
+	unsigned short usChar;
 	BOOL	bSkip;
 
 	fail(pFile == NULL);
@@ -380,8 +405,8 @@ iGetChar(FILE *pFile, list_id_enum eListID)
 	pCurr = pFontInfo;
 	bSkip = FALSE;
 	for (;;) {
-		iChar = iNextChar(pFile, eListID, &iFileOffset);
-		if (iChar == EOF) {
+		usChar = usNextChar(pFile, eListID, &lFileOffset);
+		if (usChar == (unsigned short)EOF) {
 			return EOF;
 		}
 
@@ -389,42 +414,47 @@ iGetChar(FILE *pFile, list_id_enum eListID)
 
 		if (!bStartRow) {
 			bStartRow = bRowInfo &&
-				iFileOffset == tRowInfo.iOffsetStart;
-			NO_DBG_HEX_C(bStartRow, tRowInfo.iOffsetStart);
+				lFileOffset == tRowInfo.lOffsetStart;
+			NO_DBG_HEX_C(bStartRow, tRowInfo.lOffsetStart);
 		}
 		if (!bEndRow) {
 			bEndRow = bRowInfo &&
-				iFileOffset == tRowInfo.iOffsetEnd;
-			NO_DBG_HEX_C(bStartRow, tRowInfo.iOffsetEnd);
+				lFileOffset == tRowInfo.lOffsetEnd;
+			NO_DBG_HEX_C(bStartRow, tRowInfo.lOffsetEnd);
 		}
 		if (!bStartStyle) {
 			bStartStyle = pStyleInfo != NULL &&
-				iFileOffset == pStyleInfo->iOffset;
-			NO_DBG_HEX_C(bStartStyle, iFileOffset);
+				lFileOffset == pStyleInfo->lFileOffset;
+			NO_DBG_HEX_C(bStartStyle, lFileOffset);
 		}
-		if (pCurr != NULL && iFileOffset == pCurr->iOffset) {
+		if (pCurr != NULL && lFileOffset == pCurr->lFileOffset) {
 			bStartFont = TRUE;
-			NO_DBG_HEX(iFileOffset);
+			NO_DBG_HEX(lFileOffset);
 			pFontInfo = pCurr;
 			pCurr = pGetNextFontInfoListItem(pCurr);
 		}
 
 		/* Skip embedded characters */
-		if (iChar == START_EMBEDDED) {
+		if (usChar == START_EMBEDDED) {
 			bSkip = TRUE;
 			continue;
 		}
-		if (iChar == END_IGNORE || iChar == END_EMBEDDED) {
+		if (usChar == END_IGNORE || usChar == END_EMBEDDED) {
 			bSkip = FALSE;
 			continue;
 		}
 		if (bSkip) {
 			continue;
 		}
-		iChar = iTranslateCharacters(iChar, iFileOffset, bMacFile);
-		if (iChar == EOF) {
+		iChar = iTranslateCharacters(usChar, lFileOffset, bMacFile);
+		if (iChar == IGNORE_CHARACTER) {
 			continue;
 		}
+		if (iChar == PICTURE) {
+			lFileOffsetImage = lGetPicInfoListItem(lFileOffset);
+		} else {
+			lFileOffsetImage = -1;
+		}
 		return iChar;
 	}
 } /* end of iGetChar */
@@ -436,15 +466,19 @@ void
 vWord2Text(diagram_type *pDiag, const char *szFilename)
 {
 	options_type	tOptions;
+	imagedata_type	tImage;
 	FILE	*pFile;
 	output_type	*pAnchor, *pOutput, *pLeftOver;
+	long	lWidthCurr, lWidthMax, lRightIndentation;
+	long	lDefaultTabWidth, lTmp;
 	list_id_enum 	eListID;
-	int	iFontsize, iDefaultTabWidth;
+	image_info_enum	eRes;
 	int	iFootnoteNumber, iEndnoteNumber, iLeftIndent;
-	int	iChar, iListNumber, iWidthCurr, iWidthMax, iTmp;
+	int	iChar, iListNumber;
 	BOOL	bWasTableRow, bTableFontClosed, bUnmarked;
-	BOOL	bAllCapitals, bHiddenText;
-	unsigned char	ucFontnumber, ucFontcolour, ucTmp;
+	BOOL	bAllCapitals, bHiddenText, bSuccess;
+	short	sFontsize;
+	unsigned char	ucFontnumber, ucFontcolor, ucTmp;
 	unsigned char	ucFontstyle, ucFontstyleMinimal;
 	unsigned char	ucListType, ucAlignment;
 	char	cListChar;
@@ -460,16 +494,18 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 	vAddFonts2Diagram(pDiag);
 
 	/* Initialisation */
-	uiCharCounter = 0;
+#if defined(__riscos)
+	tCharCounter = 0;
 	iCurrPct = 0;
 	iPrevPct = -1;
-	uiDocumentLength = uiGetDocumentLength();
+	tDocumentLength = tGetDocumentLength();
+#endif /* __riscos */
 	bMacFile = bFileFromTheMac();
-	iDefaultTabWidth = iGetDefaultTabWidth();
-	DBG_DEC_C(iDefaultTabWidth != 36000, iDefaultTabWidth);
+	lDefaultTabWidth = lGetDefaultTabWidth();
+	DBG_DEC_C(lDefaultTabWidth != 36000, lDefaultTabWidth);
 	bRowInfo = bGetNextRowInfoListItem(&tRowInfo);
-	DBG_HEX_C(bRowInfo, tRowInfo.iOffsetStart);
-	DBG_HEX_C(bRowInfo, tRowInfo.iOffsetEnd);
+	DBG_HEX_C(bRowInfo, tRowInfo.lOffsetStart);
+	DBG_HEX_C(bRowInfo, tRowInfo.lOffsetEnd);
 	bStartRow = FALSE;
 	bEndRow = FALSE;
 	bIsTableRow = FALSE;
@@ -479,23 +515,23 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 	bStartStyle = FALSE;
 	pAnchor = NULL;
 	pFontInfo = pGetNextFontInfoListItem(NULL);
-	DBG_HEX_C(pFontInfo != NULL, pFontInfo->iOffset);
+	DBG_HEX_C(pFontInfo != NULL, pFontInfo->lFileOffset);
 	DBG_MSG_C(pFontInfo == NULL, "No fonts at all");
 	bStartFont = FALSE;
 	ucFontnumber = 0;
 	ucFontstyleMinimal = FONT_REGULAR;
 	ucFontstyle = FONT_REGULAR;
-	iFontsize = DEFAULT_FONT_SIZE;
-	ucFontcolour = 0;
+	sFontsize = DEFAULT_FONT_SIZE;
+	ucFontcolor = FONT_COLOR_DEFAULT;
 	pAnchor = pStartNewOutput(pAnchor, NULL);
 	pOutput = pAnchor;
-	pOutput->iColour = ucFontcolour;
+	pOutput->iColor = ucFontcolor;
 	pOutput->ucFontstyle = ucFontstyle;
-	pOutput->tFontRef =
-		tOpenFont(pDiag, ucFontnumber, ucFontstyle, iFontsize);
-	pOutput->ucFontsize = iFontsize;
+	pOutput->tFontRef = tOpenFont(ucFontnumber, ucFontstyle, sFontsize);
+	pOutput->sFontsize = sFontsize;
 	bTableFontClosed = TRUE;
 	iLeftIndent = 0;
+	lRightIndentation = 0;
 	bUnmarked = TRUE;
 	ucListType = LIST_BULLETS;
 	cListChar = OUR_BULLET;
@@ -506,15 +542,15 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 	vGetOptions(&tOptions);
 	fail(tOptions.iParagraphBreak < 0);
 	if (tOptions.iParagraphBreak == 0) {
-		iWidthMax = INT_MAX;
+		lWidthMax = LONG_MAX;
 	} else if (tOptions.iParagraphBreak < MIN_SCREEN_WIDTH) {
-		iWidthMax = iChar2MilliPoints(MIN_SCREEN_WIDTH);
+		lWidthMax = lChar2MilliPoints(MIN_SCREEN_WIDTH);
 	} else if (tOptions.iParagraphBreak > MAX_SCREEN_WIDTH) {
-		iWidthMax = iChar2MilliPoints(MAX_SCREEN_WIDTH);
+		lWidthMax = lChar2MilliPoints(MAX_SCREEN_WIDTH);
 	} else {
-		iWidthMax = iChar2MilliPoints(tOptions.iParagraphBreak);
+		lWidthMax = lChar2MilliPoints(tOptions.iParagraphBreak);
 	}
-	NO_DBG_DEC(iWidthMax);
+	NO_DBG_DEC(lWidthMax);
 
 	visdelay_begin();
 
@@ -586,15 +622,14 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 				 * proportional fonts for its tables and we
 				 * only one fixed-width font
 				 */
-				pOutput->ucFontsize =
-					iFontsize <= DEFAULT_FONT_SIZE ?
-					(DEFAULT_FONT_SIZE * 5 + 3) / 6 :
-					(iFontsize * 5 + 3) / 6;
+				pOutput->sFontsize =
+					(sFontsize <= DEFAULT_FONT_SIZE ?
+					 (DEFAULT_FONT_SIZE * 5 + 3) / 6 :
+					 (sFontsize * 5 + 3) / 6);
 				pOutput->tFontRef =
-					tOpenTableFont(pDiag,
-						pOutput->ucFontsize);
+					tOpenTableFont(pOutput->sFontsize);
 				pOutput->ucFontstyle = FONT_REGULAR;
-				pOutput->iColour = FONT_COLOUR_BLACK;
+				pOutput->iColor = FONT_COLOR_BLACK;
 				bTableFontClosed = FALSE;
 			}
 			bIsTableRow = TRUE;
@@ -614,7 +649,7 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 			OUTPUT_LINE();
 			vEndOfParagraph2Diagram(pDiag,
 						pOutput->tFontRef,
-						pOutput->ucFontsize);
+						pOutput->sFontsize);
 		}
 
 		switch (iChar) {
@@ -632,11 +667,11 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 			if (iChar == FORM_FEED) {
 				vEndOfPage2Diagram(pDiag,
 						pOutput->tFontRef,
-						pOutput->ucFontsize);
+						pOutput->sFontsize);
 			} else {
 				vEndOfParagraph2Diagram(pDiag,
 						pOutput->tFontRef,
-						pOutput->ucFontsize);
+						pOutput->sFontsize);
 			}
 			break;
 		default:
@@ -651,29 +686,28 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 			ucTmp = pFontInfo->ucFontstyle &
 			(FONT_BOLD|FONT_ITALIC|FONT_UNDERLINE|FONT_STRIKE);
 			if (!bIsTableRow &&
-			    (iFontsize != pFontInfo->ucFontsize ||
+			    (sFontsize != pFontInfo->sFontsize ||
 			    ucFontnumber != pFontInfo->ucFontnumber ||
 			    ucFontstyleMinimal != ucTmp ||
-			    ucFontcolour != pFontInfo->ucFontcolour)) {
+			    ucFontcolor != pFontInfo->ucFontcolor)) {
 				pOutput = pStartNextOutput(pOutput);
 				vCloseFont();
-				pOutput->iColour = pFontInfo->ucFontcolour;
+				pOutput->iColor = pFontInfo->ucFontcolor;
 				pOutput->ucFontstyle = pFontInfo->ucFontstyle;
-				pOutput->ucFontsize = pFontInfo->ucFontsize;
+				pOutput->sFontsize = pFontInfo->sFontsize;
 				pOutput->tFontRef = tOpenFont(
-						pDiag,
 						pFontInfo->ucFontnumber,
 						pFontInfo->ucFontstyle,
-						pFontInfo->ucFontsize);
+						pFontInfo->sFontsize);
 				fail(!bCheckDoubleLinkedList(pAnchor));
 			}
 			ucFontnumber = pFontInfo->ucFontnumber;
-			iFontsize = pFontInfo->ucFontsize;
-			ucFontcolour = pFontInfo->ucFontcolour;
+			sFontsize = pFontInfo->sFontsize;
+			ucFontcolor = pFontInfo->ucFontcolor;
 			ucFontstyle = pFontInfo->ucFontstyle;
 			ucFontstyleMinimal = ucTmp;
 			pFontInfo = pGetNextFontInfoListItem(pFontInfo);
-			NO_DBG_HEX_C(pFontInfo != NULL, pFontInfo->iOffset);
+			NO_DBG_HEX_C(pFontInfo != NULL, pFontInfo->lFileOffset);
 			DBG_MSG_C(pFontInfo == NULL, "No more fonts");
 			bStartFont = FALSE;
 		}
@@ -682,13 +716,15 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 			/* Begin of a style found */
 			fail(pStyleInfo == NULL);
 			if (!bIsTableRow) {
-				vStoreStyle(pStyleInfo, pOutput);
+				vStoreStyle(pOutput);
 			}
 			iLeftIndent = pStyleInfo->sLeftIndent;
+			lRightIndentation =
+				lTwips2MilliPoints(pStyleInfo->sRightIndent);
 			bUnmarked = !pStyleInfo->bInList ||
 						pStyleInfo->bUnmarked;
 			ucListType = pStyleInfo->ucListType;
-			cListChar = pStyleInfo->ucListCharacter;
+			cListChar = (char)pStyleInfo->ucListCharacter;
 			ucAlignment = pStyleInfo->ucAlignment;
 			if (pStyleInfo->bInList) {
 				if (!pStyleInfo->bUnmarked) {
@@ -698,12 +734,13 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 				iListNumber = 0;
 			}
 			pStyleInfo = pGetNextStyleInfoListItem();
-			NO_DBG_HEX_C(pStyleInfo != NULL,pStyleInfo->iOffset);
+			NO_DBG_HEX_C(pStyleInfo != NULL,
+						pStyleInfo->lFileOffset);
 			bStartStyle = FALSE;
 		}
 
 		if (!bIsTableRow &&
-		    iTotalStringWidth(pAnchor) == 0) {
+		    lTotalStringWidth(pAnchor) == 0) {
 			vPutIndentation(pDiag, pAnchor, bUnmarked,
 					iListNumber, ucListType, cListChar,
 					iLeftIndent);
@@ -713,7 +750,33 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 
 		switch (iChar) {
 		case PICTURE:
-			vStoreString("[pic]", 5, pOutput);
+			(void)memset(&tImage, 0, sizeof(tImage));
+			eRes = eExamineImage(pFile, lFileOffsetImage, &tImage);
+			switch (eRes) {
+			case image_no_information:
+				bSuccess = FALSE;
+				break;
+			case image_minimal_information:
+			case image_full_information:
+#if 0
+				if (bOutputContainsText(pAnchor)) {
+					OUTPUT_LINE();
+				} else {
+					RESET_LINE();
+				}
+#endif
+				bSuccess = bTranslateImage(pDiag, pFile,
+					eRes == image_minimal_information,
+					lFileOffsetImage, &tImage);
+				break;
+			default:
+				DBG_DEC(eRes);
+				bSuccess = FALSE;
+				break;
+			}
+			if (!bSuccess) {
+				vStoreString("[pic]", 5, pOutput);
+			}
 			break;
 		case FOOTNOTE_CHAR:
 			iFootnoteNumber++;
@@ -742,12 +805,13 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 			}
 			vEndOfParagraph2Diagram(pDiag,
 						pOutput->tFontRef,
-						pOutput->ucFontsize);
+						pOutput->sFontsize);
 			/*
 			 * End of paragraph seen, reset indentation,
 			 * marking and alignment
 			 */
 			iLeftIndent = 0;
+			lRightIndentation = 0;
 			bUnmarked = TRUE;
 			ucAlignment = ALIGNMENT_LEFT;
 			break;
@@ -761,7 +825,7 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 			}
 			vEmptyLine2Diagram(pDiag,
 					pOutput->tFontRef,
-					pOutput->ucFontsize);
+					pOutput->sFontsize);
 			break;
 		case FORM_FEED:
 		case COLUMN_FEED:
@@ -786,16 +850,16 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 				vStoreCharacter(TAB, pOutput);
 				break;
 			}
-			iTmp = iTotalStringWidth(pAnchor);
-			iTmp += iDrawUnits2MilliPoints(pDiag->iXleft);
-			iTmp /= iDefaultTabWidth;
+			lTmp = lTotalStringWidth(pAnchor);
+			lTmp += lDrawUnits2MilliPoints(pDiag->lXleft);
+			lTmp /= lDefaultTabWidth;
 			do {
 				vStoreCharacter(FILLER_CHAR, pOutput);
-				iWidthCurr = iTotalStringWidth(pAnchor);
-				iWidthCurr +=
-					iDrawUnits2MilliPoints(pDiag->iXleft);
-			} while (iTmp == iWidthCurr / iDefaultTabWidth &&
-				 iWidthCurr < iWidthMax);
+				lWidthCurr = lTotalStringWidth(pAnchor);
+				lWidthCurr +=
+					lDrawUnits2MilliPoints(pDiag->lXleft);
+			} while (lTmp == lWidthCurr / lDefaultTabWidth &&
+				 lWidthCurr < lWidthMax + lRightIndentation);
 			break;
 		default:
 			if (bHiddenText && tOptions.bHideHiddenText) {
@@ -813,11 +877,11 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 			NO_DBG_MSG("End of table font");
 			vCloseFont();
 			bTableFontClosed = TRUE;
-			pOutput->iColour = ucFontcolour;
+			pOutput->iColor = ucFontcolor;
 			pOutput->ucFontstyle = ucFontstyle;
-			pOutput->ucFontsize = iFontsize;
-			pOutput->tFontRef = tOpenFont(pDiag, ucFontnumber,
-						ucFontstyle, iFontsize);
+			pOutput->sFontsize = sFontsize;
+			pOutput->tFontRef =
+				tOpenFont(ucFontnumber, ucFontstyle, sFontsize);
 		}
 		bWasTableRow = bIsTableRow;
 
@@ -835,26 +899,27 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 			bRowInfo = bGetNextRowInfoListItem(&tRowInfo);
 			bIsTableRow = FALSE;
 			bEndRow = FALSE;
-			NO_DBG_HEX_C(bRowInfo, tRowInfo.iOffsetStart);
-			NO_DBG_HEX_C(bRowInfo, tRowInfo.iOffsetEnd);
+			NO_DBG_HEX_C(bRowInfo, tRowInfo.lOffsetStart);
+			NO_DBG_HEX_C(bRowInfo, tRowInfo.lOffsetEnd);
 			continue;
 		}
-		iWidthCurr = iTotalStringWidth(pAnchor);
-		iWidthCurr += iDrawUnits2MilliPoints(pDiag->iXleft);
-		if (iWidthCurr < iWidthMax) {
+		lWidthCurr = lTotalStringWidth(pAnchor);
+		lWidthCurr += lDrawUnits2MilliPoints(pDiag->lXleft);
+		if (lWidthCurr < lWidthMax + lRightIndentation) {
 			continue;
 		}
 		pLeftOver = pSplitList(pAnchor);
-		vJustify2Window(pDiag, pAnchor, iWidthMax, ucAlignment);
+		vJustify2Window(pDiag, pAnchor,
+				lWidthMax, lRightIndentation, ucAlignment);
 		pAnchor = pStartNewOutput(pAnchor, pLeftOver);
 		for (pOutput = pAnchor;
 		     pOutput->pNext != NULL;
 		     pOutput = pOutput->pNext)
 			;	/* EMPTY */
 		fail(pOutput == NULL);
-		if (iTotalStringWidth(pAnchor) > 0) {
+		if (lTotalStringWidth(pAnchor) > 0) {
 			vSetLeftIndentation(pDiag,
-					iTwips2MilliPoints(iLeftIndent));
+					lTwips2MilliPoints(iLeftIndent));
 		}
 	}
 
