@@ -1,6 +1,6 @@
 /*
  * wordlib.c
- * Copyright (C) 1998,1999 A.J. van Os
+ * Copyright (C) 1998-2000 A.J. van Os; Released under GPL
  *
  * Description:
  * Deal with the internals of a MS Word file
@@ -18,7 +18,7 @@ typedef struct pps_entry_tag {
 	int	iPrev;
 	int	iDir;
 	int	iSb;
-	int	iSize;
+	long	lSize;
 	int	iLevel;
 } pps_entry_type;
 
@@ -76,122 +76,134 @@ vName2String(char *szName, const unsigned char *aucBytes, int iNameSize)
 } /* end of vName2String */
 
 /*
- * iReadBlockIndices - read the Big/Small Block Depot indices
+ * tReadBlockIndices - read the Big/Small Block Depot indices
  *
- * returns the number of indices read
+ * Returns the number of indices read
  */
-static int
-iReadBlockIndices(FILE *pFile, int *aiBlockDepot, int iMaxRec, int iOffset)
+static size_t
+tReadBlockIndices(FILE *pFile, int *aiBlockDepot, size_t tMaxRec, long lOffset)
 {
-	int	iIndex, iDone;
+	size_t	tDone;
+	int	iIndex;
 	unsigned char	aucBytes[BIG_BLOCK_SIZE];
 
 	fail(pFile == NULL || aiBlockDepot == NULL);
-	fail(iMaxRec < 0 || iOffset < 0);
+	fail(tMaxRec == 0);
+	fail(lOffset < 0);
 
 	/* Read a big block with BBD or SBD indices */
-	if (!bReadBytes(aucBytes, BIG_BLOCK_SIZE, iOffset, pFile)) {
-		werr(0, "Reading big block from %d is not possible", iOffset);
-		return -1;
+	if (!bReadBytes(aucBytes, BIG_BLOCK_SIZE, lOffset, pFile)) {
+		werr(0, "Reading big block from %ld is not possible", lOffset);
+		return 0;
 	}
 	/* Split the big block into indices, an index is four bytes */
-	iDone = min(iMaxRec, BIG_BLOCK_SIZE / 4);
-	for (iIndex = 0; iIndex < iDone; iIndex++) {
+	tDone = min(tMaxRec, BIG_BLOCK_SIZE / 4);
+	for (iIndex = 0; iIndex < (int)tDone; iIndex++) {
 		aiBlockDepot[iIndex] = (int)ulGetLong(4 * iIndex, aucBytes);
 		NO_DBG_DEC(aiBlockDepot[iIndex]);
 	}
-	return iDone;
-} /* end of iReadBlockIndices */
+	return tDone;
+} /* end of tReadBlockIndices */
 
 /*
  * bGetBBD - get the Big Block Depot indices from the index-blocks
  */
 static BOOL
-bGetBBD(FILE *pFile, const int *aiDepot, int iDepotLen,
-			int *aiBBD, int iBBDLen)
+bGetBBD(FILE *pFile, const int *aiDepot, size_t tDepotLen,
+			int *aiBBD, size_t tBBDLen)
 {
-	int	iIndex, iBegin, iToGo, iDone;
+	long	lBegin;
+	size_t	tToGo, tDone;
+	int	iIndex;
 
 	fail(pFile == NULL || aiDepot == NULL || aiBBD == NULL);
-	fail(iDepotLen < 0 || iBBDLen < 0);
 
 	DBG_MSG("bGetBBD");
 
-	iToGo = iBBDLen;
-	for (iIndex = 0; iIndex < iDepotLen && iToGo > 0; iIndex++) {
-		iBegin = (aiDepot[iIndex] + 1) * BIG_BLOCK_SIZE;
-		NO_DBG_HEX(iBegin);
-		iDone = iReadBlockIndices(pFile, aiBBD, iToGo, iBegin);
-		if (iDone < 0) {
+	tToGo = tBBDLen;
+	for (iIndex = 0; iIndex < (int)tDepotLen && tToGo != 0; iIndex++) {
+		lBegin = ((long)aiDepot[iIndex] + 1) * BIG_BLOCK_SIZE;
+		NO_DBG_HEX(lBegin);
+		tDone = tReadBlockIndices(pFile, aiBBD, tToGo, lBegin);
+		fail(tDone > tToGo);
+		if (tDone == 0) {
 			return FALSE;
 		}
-		aiBBD += iDone;
-		iToGo -= iDone;
+		aiBBD += tDone;
+		tToGo -= tDone;
 	}
-	return iToGo <= 0;
+	return tToGo == 0;
 } /* end of bGetBBD */
 
 /*
  * bGetSBD - get the Small Block Depot indices from the index-blocks
  */
 static BOOL
-bGetSBD(FILE *pFile, const int *aiDepot, int iDepotLen,
-			int *aiSBD, int iSBDLen)
+bGetSBD(FILE *pFile, const int *aiDepot, size_t tDepotLen,
+			int *aiSBD, size_t tSBDLen)
 {
-	int	iIndex, iBegin, iToGo, iDone;
+	long	lBegin;
+	size_t	tToGo, tDone;
+	int	iIndex;
 
 	fail(pFile == NULL || aiDepot == NULL || aiSBD == NULL);
-	fail(iDepotLen < 0 || iSBDLen < 0);
 
 	DBG_MSG("bGetSBD");
 
-	iToGo = iSBDLen;
-	for (iIndex = 0; iIndex < iDepotLen && iToGo > 0; iIndex++) {
-		iBegin = (aiDepot[iIndex] + 1) * BIG_BLOCK_SIZE;
-		NO_DBG_HEX(iBegin);
-		iDone = iReadBlockIndices(pFile, aiSBD, iToGo, iBegin);
-		if (iDone < 0) {
+	tToGo = tSBDLen;
+	for (iIndex = 0; iIndex < (int)tDepotLen && tToGo != 0; iIndex++) {
+		lBegin = ((long)aiDepot[iIndex] + 1) * BIG_BLOCK_SIZE;
+		NO_DBG_HEX(lBegin);
+		tDone = tReadBlockIndices(pFile, aiSBD, tToGo, lBegin);
+		fail(tDone > tToGo);
+		if (tDone == 0) {
 			return FALSE;
 		}
-		aiSBD += iDone;
-		iToGo -= iDone;
+		aiSBD += tDone;
+		tToGo -= tDone;
 	}
-	return iToGo <= 0;
+	return tToGo == 0;
 } /* end of bGetSBD */
 
 /*
  * vComputePPSlevels - compute the levels of the Property Set Storage entries
  */
 static void
-vComputePPSlevels(pps_entry_type *atPPSlist, pps_entry_type *ptNode,
+vComputePPSlevels(pps_entry_type *atPPSlist, pps_entry_type *pNode,
 			int iLevel, int iRecursionLevel)
 {
-	fail(atPPSlist == NULL || ptNode == NULL);
+	fail(atPPSlist == NULL || pNode == NULL);
 	fail(iLevel < 0 || iRecursionLevel < 0);
 
-	if (iRecursionLevel > 50) {
-		/* Remove the possibility of an infinite recursion */
+	if (iRecursionLevel > 25) {
+		/* This removes the possibility of an infinite recursion */
 		DBG_DEC(iRecursionLevel);
 		return;
 	}
+	if (pNode->iLevel <= iLevel) {
+		/* Avoid entering a loop */
+		DBG_DEC(iLevel);
+		DBG_DEC(pNode->iLevel);
+		return;
+	}
 
-	ptNode->iLevel = iLevel;
+	pNode->iLevel = iLevel;
 
-	if (ptNode->iDir != -1) {
+	if (pNode->iDir != -1) {
 		vComputePPSlevels(atPPSlist,
-				&atPPSlist[ptNode->iDir],
+				&atPPSlist[pNode->iDir],
 				iLevel + 1,
 				iRecursionLevel + 1);
 	}
-	if (ptNode->iNext != -1) {
+	if (pNode->iNext != -1) {
 		vComputePPSlevels(atPPSlist,
-				&atPPSlist[ptNode->iNext],
+				&atPPSlist[pNode->iNext],
 				iLevel,
 				iRecursionLevel + 1);
 	}
-	if (ptNode->iPrev != -1) {
+	if (pNode->iPrev != -1) {
 		vComputePPSlevels(atPPSlist,
-				&atPPSlist[ptNode->iPrev],
+				&atPPSlist[pNode->iPrev],
 				iLevel,
 				iRecursionLevel + 1);
 	}
@@ -204,38 +216,39 @@ vComputePPSlevels(pps_entry_type *atPPSlist, pps_entry_type *ptNode,
  */
 static BOOL
 bGetPPS(FILE *pFile,
-		const int *aiRootList, int iRootListLen,
+		const int *aiRootList, size_t tRootListLen,
 		pps_info_type *pPPS)
 {
 	pps_entry_type	*atPPSlist;
-	int	iTmp, iIndex, iStartBlock, iOffset, iBegin;
-	int	iNameSize, iRootIndex, iNbrOfPPS;
+	long	lBegin;
+	size_t	tNbrOfPPS;
+	int	iTmp, iIndex, iStartBlock, iOffset;
+	int	iNameSize, iRootIndex;
 	BOOL	bWord, bExcel;
 	unsigned char	aucBytes[PROPERTY_SET_STORAGE_SIZE];
 
-	fail(pFile == NULL || pPPS == NULL);
-	fail(aiRootList == NULL || iRootListLen < 0);
+	fail(pFile == NULL || pPPS == NULL || aiRootList == NULL);
 
 	DBG_MSG("bGetPPS");
-	NO_DBG_DEC(iRootListLen);
+	NO_DBG_DEC(tRootListLen);
 
 	bWord = FALSE;
 	bExcel = FALSE;
 	(void)memset(pPPS, 0, sizeof(*pPPS));
 
 	/* Read and store all the Property Set Storage entries */
-	iNbrOfPPS = iRootListLen * BIG_BLOCK_SIZE / PROPERTY_SET_STORAGE_SIZE;
-	atPPSlist = xmalloc(iNbrOfPPS * sizeof(pps_entry_type));
+	tNbrOfPPS = tRootListLen * BIG_BLOCK_SIZE / PROPERTY_SET_STORAGE_SIZE;
+	atPPSlist = xmalloc(tNbrOfPPS * sizeof(pps_entry_type));
 	iRootIndex = 0;
-	for (iIndex = 0; iIndex < iNbrOfPPS; iIndex++) {
+	for (iIndex = 0; iIndex < (int)tNbrOfPPS; iIndex++) {
 		iTmp = iIndex * PROPERTY_SET_STORAGE_SIZE;
 		iStartBlock = iTmp / BIG_BLOCK_SIZE;
 		iOffset = iTmp % BIG_BLOCK_SIZE;
-		iBegin = (aiRootList[iStartBlock] + 1) * BIG_BLOCK_SIZE +
+		lBegin = ((long)aiRootList[iStartBlock] + 1) * BIG_BLOCK_SIZE +
 			iOffset;
-		NO_DBG_HEX(iBegin);
+		NO_DBG_HEX(lBegin);
 		if (!bReadBytes(aucBytes, PROPERTY_SET_STORAGE_SIZE,
-							iBegin, pFile)) {
+							lBegin, pFile)) {
 			werr(0, "Reading PPS %d is not possible", iIndex);
 			atPPSlist = xfree(atPPSlist);
 			return FALSE;
@@ -251,14 +264,19 @@ bGetPPS(FILE *pFile,
 		atPPSlist[iIndex].iNext = (int)ulGetLong(0x48, aucBytes);
 		atPPSlist[iIndex].iDir = (int)ulGetLong(0x4c, aucBytes);
 		atPPSlist[iIndex].iSb = (int)ulGetLong(0x74, aucBytes);
-		atPPSlist[iIndex].iSize = (int)ulGetLong(0x78, aucBytes);
-		atPPSlist[iIndex].iLevel = -1;
+		atPPSlist[iIndex].lSize = (long)ulGetLong(0x78, aucBytes);
+		atPPSlist[iIndex].iLevel = INT_MAX;
 		if (atPPSlist[iIndex].iPrev < -1 ||
-		    atPPSlist[iIndex].iPrev >= iNbrOfPPS ||
+		    atPPSlist[iIndex].iPrev >= (int)tNbrOfPPS ||
 		    atPPSlist[iIndex].iNext < -1 ||
-		    atPPSlist[iIndex].iNext >= iNbrOfPPS ||
+		    atPPSlist[iIndex].iNext >= (int)tNbrOfPPS ||
 		    atPPSlist[iIndex].iDir < -1 ||
-		    atPPSlist[iIndex].iDir >= iNbrOfPPS) {
+		    atPPSlist[iIndex].iDir >= (int)tNbrOfPPS) {
+			DBG_DEC(iIndex);
+			DBG_DEC(atPPSlist[iIndex].iPrev);
+			DBG_DEC(atPPSlist[iIndex].iNext);
+			DBG_DEC(atPPSlist[iIndex].iDir);
+			DBG_DEC(tNbrOfPPS);
 			werr(0, "The Property Set Storage is damaged");
 			atPPSlist = xfree(atPPSlist);
 			return FALSE;
@@ -267,13 +285,13 @@ bGetPPS(FILE *pFile,
 
 #if 0 /* defined(DEBUG) */
 	DBG_MSG("Before");
-	for (iIndex = 0; iIndex < iNbrOfPPS; iIndex++) {
+	for (iIndex = 0; iIndex < (int)tNbrOfPPS; iIndex++) {
 		DBG_MSG(atPPSlist[iIndex].szName);
 		DBG_HEX(atPPSlist[iIndex].iDir);
 		DBG_HEX(atPPSlist[iIndex].iPrev);
 		DBG_HEX(atPPSlist[iIndex].iNext);
 		DBG_DEC(atPPSlist[iIndex].iSb);
-		DBG_HEX(atPPSlist[iIndex].iSize);
+		DBG_HEX(atPPSlist[iIndex].lSize);
 		DBG_DEC(atPPSlist[iIndex].iLevel);
 	}
 #endif /* DEBUG */
@@ -283,37 +301,41 @@ bGetPPS(FILE *pFile,
 
 	/* Check the entries on level 1 for the required information */
 	NO_DBG_MSG("After");
-	for (iIndex = 0; iIndex < iNbrOfPPS; iIndex++) {
+	for (iIndex = 0; iIndex < (int)tNbrOfPPS; iIndex++) {
 #if 0 /* defined(DEBUG) */
 		DBG_MSG(atPPSlist[iIndex].szName);
 		DBG_HEX(atPPSlist[iIndex].iDir);
 		DBG_HEX(atPPSlist[iIndex].iPrev);
 		DBG_HEX(atPPSlist[iIndex].iNext);
 		DBG_DEC(atPPSlist[iIndex].iSb);
-		DBG_HEX(atPPSlist[iIndex].iSize);
+		DBG_HEX(atPPSlist[iIndex].lSize);
 		DBG_DEC(atPPSlist[iIndex].iLevel);
 #endif /* DEBUG */
 		if (atPPSlist[iIndex].iLevel != 1 ||
 		    atPPSlist[iIndex].iType != 2 ||
 		    atPPSlist[iIndex].szName[0] == '\0' ||
-		    atPPSlist[iIndex].iSize <= 0) {
+		    atPPSlist[iIndex].lSize <= 0) {
 			continue;
 		}
-		if (pPPS->tWordDocument.iSize <= 0 &&
-		    StrEq(atPPSlist[iIndex].szName, "WordDocument")) {
+		if (pPPS->tWordDocument.lSize <= 0 &&
+		    STREQ(atPPSlist[iIndex].szName, "WordDocument")) {
 			pPPS->tWordDocument.iSb = atPPSlist[iIndex].iSb;
-			pPPS->tWordDocument.iSize = atPPSlist[iIndex].iSize;
+			pPPS->tWordDocument.lSize = atPPSlist[iIndex].lSize;
 			bWord = TRUE;
-		} else if (pPPS->t0Table.iSize <= 0 &&
-			   StrEq(atPPSlist[iIndex].szName, "0Table")) {
+		} else if (pPPS->tData.lSize <= 0 &&
+			   STREQ(atPPSlist[iIndex].szName, "Data")) {
+			pPPS->tData.iSb = atPPSlist[iIndex].iSb;
+			pPPS->tData.lSize = atPPSlist[iIndex].lSize;
+		} else if (pPPS->t0Table.lSize <= 0 &&
+			   STREQ(atPPSlist[iIndex].szName, "0Table")) {
 			pPPS->t0Table.iSb = atPPSlist[iIndex].iSb;
-			pPPS->t0Table.iSize = atPPSlist[iIndex].iSize;
-		} else if (pPPS->t1Table.iSize <= 0 &&
-			   StrEq(atPPSlist[iIndex].szName, "1Table")) {
+			pPPS->t0Table.lSize = atPPSlist[iIndex].lSize;
+		} else if (pPPS->t1Table.lSize <= 0 &&
+			   STREQ(atPPSlist[iIndex].szName, "1Table")) {
 			pPPS->t1Table.iSb = atPPSlist[iIndex].iSb;
-			pPPS->t1Table.iSize = atPPSlist[iIndex].iSize;
-		} else if (StrEq(atPPSlist[iIndex].szName, "Book") ||
-			   StrEq(atPPSlist[iIndex].szName, "Workbook")) {
+			pPPS->t1Table.lSize = atPPSlist[iIndex].lSize;
+		} else if (STREQ(atPPSlist[iIndex].szName, "Book") ||
+			   STREQ(atPPSlist[iIndex].szName, "Workbook")) {
 			bExcel = TRUE;
 		}
 	}
@@ -335,23 +357,26 @@ bGetPPS(FILE *pFile,
 
 /*
  * bGetDocumentText - make a list of the text blocks of a Word document
+ *
+ * Return TRUE when succesful, otherwise FALSE
  */
 static BOOL
 bGetDocumentText(FILE *pFile, const pps_info_type *pPPS,
-		const int *aiBBD, int iBBDLen,
-		const int *aiSBD, int iSBDLen,
+		const int *aiBBD, size_t tBBDLen,
+		const int *aiSBD, size_t tSBDLen,
 		const unsigned char *aucHeader, int iWordVersion)
 {
-	int	iBeginOfText;
-	int	iTextLen, iFootnoteLen, iEndnoteLen;
-	int	iHeaderLen, iMacroLen, iAnnotationLen;
-	int	iTextBoxLen, iHdrTextBoxLen;
+	long	lBeginOfText;
+	size_t	tTextLen, tFootnoteLen, tEndnoteLen;
+	size_t	tHeaderLen, tMacroLen, tAnnotationLen;
+	size_t	tTextBoxLen, tHdrTextBoxLen;
 	BOOL	bFarEastWord, bFastSaved, bEncrypted, bSuccess;
 	text_info_enum	eResult;
 	unsigned short	usDocStatus, usIdent;
 
 	fail(pFile == NULL || pPPS == NULL);
-	fail(aiBBD == NULL || iBBDLen < 0);
+	fail(aiBBD == NULL);
+	fail(aiSBD == NULL);
 
 	DBG_MSG("bGetDocumentText");
 
@@ -365,7 +390,7 @@ bGetDocumentText(FILE *pFile, const pps_info_type *pPPS,
 	DBG_HEX(usDocStatus);
 	bFastSaved = (usDocStatus & BIT(2)) != 0;
 	DBG_MSG_C(bFastSaved, "This document is Fast Saved");
-	DBG_DEC_C(bFastSaved, (usDocStatus & 0xf0) >> 4);
+	DBG_DEC_C(bFastSaved, (usDocStatus & 0x00f0) >> 4);
 	bEncrypted = (usDocStatus & BIT(8)) != 0;
 	if (bEncrypted) {
 		werr(0, "Encrypted documents are not supported");
@@ -373,35 +398,35 @@ bGetDocumentText(FILE *pFile, const pps_info_type *pPPS,
 	}
 
 	/* Get length information */
-	iBeginOfText = (int)ulGetLong(0x18, aucHeader);
-	DBG_HEX(iBeginOfText);
+	lBeginOfText = (long)ulGetLong(0x18, aucHeader);
+	DBG_HEX(lBeginOfText);
 	if (iWordVersion == 6 || iWordVersion == 7) {
-		iTextLen = (int)ulGetLong(0x34, aucHeader);
-		iFootnoteLen = (int)ulGetLong(0x38, aucHeader);
-		iHeaderLen = (int)ulGetLong(0x3c, aucHeader);
-		iMacroLen = (int)ulGetLong(0x40, aucHeader);
-		iAnnotationLen = (int)ulGetLong(0x44, aucHeader);
-		iEndnoteLen = (int)ulGetLong(0x48, aucHeader);
-		iTextBoxLen = (int)ulGetLong(0x4c, aucHeader);
-		iHdrTextBoxLen = (int)ulGetLong(0x50, aucHeader);
+		tTextLen = (size_t)ulGetLong(0x34, aucHeader);
+		tFootnoteLen = (size_t)ulGetLong(0x38, aucHeader);
+		tHeaderLen = (size_t)ulGetLong(0x3c, aucHeader);
+		tMacroLen = (size_t)ulGetLong(0x40, aucHeader);
+		tAnnotationLen = (size_t)ulGetLong(0x44, aucHeader);
+		tEndnoteLen = (size_t)ulGetLong(0x48, aucHeader);
+		tTextBoxLen = (size_t)ulGetLong(0x4c, aucHeader);
+		tHdrTextBoxLen = (size_t)ulGetLong(0x50, aucHeader);
 	} else {
-		iTextLen = (int)ulGetLong(0x4c, aucHeader);
-		iFootnoteLen = (int)ulGetLong(0x50, aucHeader);
-		iHeaderLen = (int)ulGetLong(0x54, aucHeader);
-		iMacroLen = (int)ulGetLong(0x58, aucHeader);
-		iAnnotationLen = (int)ulGetLong(0x5c, aucHeader);
-		iEndnoteLen = (int)ulGetLong(0x60, aucHeader);
-		iTextBoxLen = (int)ulGetLong(0x64, aucHeader);
-		iHdrTextBoxLen = (int)ulGetLong(0x68, aucHeader);
+		tTextLen = (size_t)ulGetLong(0x4c, aucHeader);
+		tFootnoteLen = (size_t)ulGetLong(0x50, aucHeader);
+		tHeaderLen = (size_t)ulGetLong(0x54, aucHeader);
+		tMacroLen = (size_t)ulGetLong(0x58, aucHeader);
+		tAnnotationLen = (size_t)ulGetLong(0x5c, aucHeader);
+		tEndnoteLen = (size_t)ulGetLong(0x60, aucHeader);
+		tTextBoxLen = (size_t)ulGetLong(0x64, aucHeader);
+		tHdrTextBoxLen = (size_t)ulGetLong(0x68, aucHeader);
 	}
-	DBG_DEC(iTextLen);
-	DBG_DEC(iFootnoteLen);
-	DBG_DEC(iHeaderLen);
-	DBG_DEC(iMacroLen);
-	DBG_DEC(iAnnotationLen);
-	DBG_DEC(iEndnoteLen);
-	DBG_DEC(iTextBoxLen);
-	DBG_DEC(iHdrTextBoxLen);
+	DBG_DEC(tTextLen);
+	DBG_DEC(tFootnoteLen);
+	DBG_DEC(tHeaderLen);
+	DBG_DEC(tMacroLen);
+	DBG_DEC(tAnnotationLen);
+	DBG_DEC(tEndnoteLen);
+	DBG_DEC(tTextBoxLen);
+	DBG_DEC(tHdrTextBoxLen);
 
 	/* Make a list of the text blocks */
 	switch (iWordVersion) {
@@ -411,37 +436,37 @@ bGetDocumentText(FILE *pFile, const pps_info_type *pPPS,
 			eResult = eGet6DocumentText(pFile,
 					bFarEastWord,
 					pPPS->tWordDocument.iSb,
-					aiBBD, iBBDLen,
+					aiBBD, tBBDLen,
 					aucHeader);
 			bSuccess = eResult == text_success;
 		} else {
-		  	bSuccess = bAddBlocks(iBeginOfText,
-				iTextLen +
-				iFootnoteLen +
-				iHeaderLen + iMacroLen + iAnnotationLen +
-				iEndnoteLen +
-				iTextBoxLen + iHdrTextBoxLen,
+		  	bSuccess = bAddTextBlocks(lBeginOfText,
+				tTextLen +
+				tFootnoteLen +
+				tHeaderLen + tMacroLen + tAnnotationLen +
+				tEndnoteLen +
+				tTextBoxLen + tHdrTextBoxLen,
 				bFarEastWord,
 				pPPS->tWordDocument.iSb,
-				aiBBD, iBBDLen);
+				aiBBD, tBBDLen);
 		}
 		break;
 	case 8:
 		eResult = eGet8DocumentText(pFile,
 				pPPS,
-				aiBBD, iBBDLen, aiSBD, iSBDLen,
+				aiBBD, tBBDLen, aiSBD, tSBDLen,
 				aucHeader);
 		if (eResult == text_no_information && !bFastSaved) {
 			/* Assume there is only one block */
-			bSuccess = bAddBlocks(iBeginOfText,
-				iTextLen +
-				iFootnoteLen +
-				iHeaderLen + iMacroLen + iAnnotationLen +
-				iEndnoteLen +
-				iTextBoxLen + iHdrTextBoxLen,
+			bSuccess = bAddTextBlocks(lBeginOfText,
+				tTextLen +
+				tFootnoteLen +
+				tHeaderLen + tMacroLen + tAnnotationLen +
+				tEndnoteLen +
+				tTextBoxLen + tHdrTextBoxLen,
 				FALSE,
 				pPPS->tWordDocument.iSb,
-				aiBBD, iBBDLen);
+				aiBBD, tBBDLen);
 		} else {
 			bSuccess = eResult == text_success;
 		}
@@ -453,11 +478,11 @@ bGetDocumentText(FILE *pFile, const pps_info_type *pPPS,
 	}
 
 	if (bSuccess) {
-		vSplitBlockList(iTextLen,
-				iFootnoteLen,
-				iHeaderLen + iMacroLen + iAnnotationLen,
-				iEndnoteLen,
-				iTextBoxLen + iHdrTextBoxLen,
+		vSplitBlockList(tTextLen,
+				tFootnoteLen,
+				tHeaderLen + tMacroLen + tAnnotationLen,
+				tEndnoteLen,
+				tTextBoxLen + tHdrTextBoxLen,
 				!bFastSaved && iWordVersion == 8);
 	} else {
 		vDestroyTextBlockList();
@@ -466,6 +491,85 @@ bGetDocumentText(FILE *pFile, const pps_info_type *pPPS,
 	return bSuccess;
 } /* end of bGetDocumentText */
 
+/*
+ * vGetDocumentData - make a list of the data blocks of a Word document
+ */
+static void
+vGetDocumentData(FILE *pFile, const pps_info_type *pPPS,
+		const int *aiBBD, size_t tBBDLen,
+		const unsigned char *aucHeader, int iWordVersion)
+{
+	options_type	tOptions;
+	long	lBeginOfText;
+	BOOL	bFastSaved, bHasImages, bSuccess;
+	unsigned short	usDocStatus;
+
+	fail(pFile == NULL);
+	fail(pPPS == NULL);
+	fail(aiBBD == NULL);
+
+	/* Get the options */
+	vGetOptions(&tOptions);
+
+	/* Get the status flags from the header */
+	usDocStatus = usGetWord(0x0a, aucHeader);
+	DBG_HEX(usDocStatus);
+	bFastSaved = (usDocStatus & BIT(2)) != 0;
+	bHasImages = (usDocStatus & BIT(3)) != 0;
+
+	if (!bHasImages ||
+	    !tOptions.bUseOutlineFonts ||
+	    tOptions.eImageLevel == level_no_images) {
+		/*
+		 * No images in the document or text-only output or
+		 * no images wanted, so no data blocks will be needed
+		 */
+		vDestroyDataBlockList();
+		return;
+	}
+
+	/* Get length information */
+	lBeginOfText = (long)ulGetLong(0x18, aucHeader);
+	DBG_HEX(lBeginOfText);
+
+	/* Make a list of the data blocks */
+	switch (iWordVersion) {
+	case 6:
+	case 7:
+		/*
+		 * The data blocks are in the text stream. The text stream
+		 * is in "fast saved" format or "normal saved" format
+		 */
+		if (bFastSaved) {
+			bSuccess = bGet6DocumentData(pFile,
+					pPPS->tWordDocument.iSb,
+					aiBBD, tBBDLen,
+					aucHeader);
+		} else {
+		  	bSuccess = bAddDataBlocks(lBeginOfText, SIZE_T_MAX,
+				pPPS->tWordDocument.iSb, aiBBD, tBBDLen);
+		}
+		break;
+	case 8:
+		/*
+		 * The data blocks are in the data stream. The data stream
+		 * is always in "normal saved" format
+		 */
+		bSuccess = bAddDataBlocks(0, SIZE_T_MAX,
+				pPPS->tData.iSb, aiBBD, tBBDLen);
+		break;
+	default:
+		werr(0, "This version of Word is not supported");
+		bSuccess = FALSE;
+		break;
+	}
+
+	if (!bSuccess) {
+		vDestroyDataBlockList();
+		werr(0, "I can't find the data of this document");
+	}
+} /* end of vGetDocumentData */
+
 /*
  * pInitFile - open and check the file
  */
@@ -485,34 +589,34 @@ pInitFile(const char *szFilename)
 } /* end of pInitFile */
 
 static BOOL
-bInitDoc(FILE *pFile, off_t tFilesize)
+bInitDoc(FILE *pFile, long lFilesize)
 {
 	pps_info_type	PPS_info;
 	int	*aiBBD, *aiRootList, *aiBbdList;
 	int	*aiSBD, *aiSbdList;
-	size_t	tSize;
-	int	iRootListLen, iRootStartblock;
+	long	lMaxBlock;
+	size_t	tBBDLen, tSBDLen, tNumBbdBlocks, tRootListLen, tSize;
+	int	iRootStartblock;
 	int	iSbdStartblock, iSBLstartblock;
-	int	iBBDLen, iSBDLen;
 	int	iWordVersion, iIndex, iTmp;
-	int	iMaxBlock, iNumBbdBlocks, iMaxSmallBlock;
+	int	iMaxSmallBlock;
 	BOOL	bResult;
 	unsigned short	usIdent, usFib, usChse;
 	unsigned char	aucHeader[HEADER_SIZE];
 
 	fail(pFile == NULL);
 
-	iMaxBlock = (int)(tFilesize / BIG_BLOCK_SIZE) - 2;
-	DBG_DEC(iMaxBlock);
-	if (iMaxBlock < 1) {
+	lMaxBlock = lFilesize / BIG_BLOCK_SIZE - 2;
+	DBG_DEC(lMaxBlock);
+	if (lMaxBlock < 1) {
 		return FALSE;
 	}
-	iBBDLen = iMaxBlock + 1;
-	iNumBbdBlocks = (int)ulReadLong(pFile, 0x2c);
-	DBG_DEC(iNumBbdBlocks);
-	if (iNumBbdBlocks > 109) {
-		DBG_DEC(tFilesize);
-		if (tFilesize > 109 * (BIG_BLOCK_SIZE/4) * BIG_BLOCK_SIZE) {
+	tBBDLen = (size_t)(lMaxBlock + 1);
+	tNumBbdBlocks = (size_t)ulReadLong(pFile, 0x2c);
+	DBG_DEC(tNumBbdBlocks);
+	if (tNumBbdBlocks > 109) {
+		DBG_DEC(lFilesize);
+		if (lFilesize > 109 * (BIG_BLOCK_SIZE/4) * BIG_BLOCK_SIZE) {
 			werr(0, "I'm afraid this file is too big to handle.");
 		} else {
 			werr(0, "This file is probably damaged. Sorry.");
@@ -530,7 +634,7 @@ bInitDoc(FILE *pFile, off_t tFilesize)
 		((long)iRootStartblock + 1) *
 		BIG_BLOCK_SIZE + 0x78) / SMALL_BLOCK_SIZE) - 1;
 	DBG_DEC(iMaxSmallBlock);
-	iSBDLen = iMaxSmallBlock + 1;
+	tSBDLen = (size_t)(iMaxSmallBlock + 1);
 	/* All to be xmalloc-ed pointers to NULL */
 	aiRootList = NULL;
 	aiSbdList = NULL;
@@ -538,78 +642,78 @@ bInitDoc(FILE *pFile, off_t tFilesize)
 	aiSBD = NULL;
 	aiBBD = NULL;
 /* Big Block Depot */
-	tSize = iNumBbdBlocks * sizeof(int);
+	tSize = tNumBbdBlocks * sizeof(int);
 	aiBbdList = xmalloc(tSize);
-	tSize = iBBDLen * sizeof(int);
+	tSize = tBBDLen * sizeof(int);
 	aiBBD = xmalloc(tSize);
-	for (iIndex = 0; iIndex < iNumBbdBlocks; iIndex++) {
+	for (iIndex = 0; iIndex < (int)tNumBbdBlocks; iIndex++) {
 		aiBbdList[iIndex] =
 			(int)ulReadLong(pFile, 0x4c + 4 * (long)iIndex);
 		NO_DBG_HEX(aiBbdList[iIndex]);
 	}
-	if (!bGetBBD(pFile, aiBbdList, iNumBbdBlocks, aiBBD, iBBDLen)) {
+	if (!bGetBBD(pFile, aiBbdList, tNumBbdBlocks, aiBBD, tBBDLen)) {
 		FREE_ALL();
 		return FALSE;
 	}
 	aiBbdList = xfree(aiBbdList);
 /* Small Block Depot */
-	tSize = iBBDLen * sizeof(int);
+	tSize = tBBDLen * sizeof(int);
 	aiSbdList = xmalloc(tSize);
-	tSize = iSBDLen * sizeof(int);
+	tSize = tSBDLen * sizeof(int);
 	aiSBD = xmalloc(tSize);
 	for (iIndex = 0, iTmp = iSbdStartblock;
-	     iIndex < iBBDLen && iTmp != END_OF_CHAIN;
+	     iIndex < (int)tBBDLen && iTmp != END_OF_CHAIN;
 	     iIndex++, iTmp = aiBBD[iTmp]) {
-		if (iTmp < 0 || iTmp >= iBBDLen) {
+		if (iTmp < 0 || iTmp >= (int)tBBDLen) {
 			werr(1, "The Big Block Depot is damaged");
 		}
 		aiSbdList[iIndex] = iTmp;
 		NO_DBG_HEX(aiSbdList[iIndex]);
 	}
-	if (!bGetSBD(pFile, aiSbdList, iBBDLen, aiSBD, iSBDLen)) {
+	if (!bGetSBD(pFile, aiSbdList, tBBDLen, aiSBD, tSBDLen)) {
 		FREE_ALL();
 		return FALSE;
 	}
 	aiSbdList = xfree(aiSbdList);
 /* Root list */
-	for (iRootListLen = 0, iTmp = iRootStartblock;
-	     iRootListLen < iBBDLen && iTmp != END_OF_CHAIN;
-	     iRootListLen++, iTmp = aiBBD[iTmp]) {
-		if (iTmp < 0 || iTmp >= iBBDLen) {
+	for (tRootListLen = 0, iTmp = iRootStartblock;
+	     tRootListLen < tBBDLen && iTmp != END_OF_CHAIN;
+	     tRootListLen++, iTmp = aiBBD[iTmp]) {
+		if (iTmp < 0 || iTmp >= (int)tBBDLen) {
 			werr(1, "The Big Block Depot is damaged");
 		}
 	}
-	if (iRootListLen <= 0) {
+	if (tRootListLen == 0) {
 		werr(0, "No Rootlist found");
 		FREE_ALL();
 		return FALSE;
 	}
-	tSize = iRootListLen * sizeof(int);
+	tSize = tRootListLen * sizeof(int);
 	aiRootList = xmalloc(tSize);
 	for (iIndex = 0, iTmp = iRootStartblock;
-	     iIndex < iBBDLen && iTmp != END_OF_CHAIN;
+	     iIndex < (int)tBBDLen && iTmp != END_OF_CHAIN;
 	     iIndex++, iTmp = aiBBD[iTmp]) {
-		if (iTmp < 0 || iTmp >= iBBDLen) {
+		if (iTmp < 0 || iTmp >= (int)tBBDLen) {
 			werr(1, "The Big Block Depot is damaged");
 		}
 		aiRootList[iIndex] = iTmp;
 		NO_DBG_DEC(aiRootList[iIndex]);
 	}
-	fail(iRootListLen != iIndex);
-	bResult = bGetPPS(pFile, aiRootList, iRootListLen, &PPS_info);
+	fail(tRootListLen != (size_t)iIndex);
+	bResult = bGetPPS(pFile, aiRootList, tRootListLen, &PPS_info);
 	aiRootList = xfree(aiRootList);
 	if (!bResult) {
 		FREE_ALL();
 		return FALSE;
 	}
 /* Small block list */
-	if (!bCreateSmallBlockList(iSBLstartblock, aiBBD, iBBDLen)) {
+	if (!bCreateSmallBlockList(iSBLstartblock, aiBBD, tBBDLen)) {
 		FREE_ALL();
 		return FALSE;
 	}
 
-	if (PPS_info.tWordDocument.iSize < MIN_SIZE_FOR_BBD_USE) {
-		DBG_DEC(PPS_info.tWordDocument.iSize);
+	if (PPS_info.tWordDocument.lSize < MIN_SIZE_FOR_BBD_USE) {
+		DBG_DEC(PPS_info.tWordDocument.lSize);
 		FREE_ALL();
 		werr(0, "I'm afraid the text stream of this file "
 			"is too small to handle.");
@@ -617,7 +721,7 @@ bInitDoc(FILE *pFile, off_t tFilesize)
 	}
 	/* Read the headerblock */
 	if (!bReadBuffer(pFile, PPS_info.tWordDocument.iSb,
-			aiBBD, iBBDLen, BIG_BLOCK_SIZE,
+			aiBBD, tBBDLen, BIG_BLOCK_SIZE,
 			aucHeader, 0, HEADER_SIZE)) {
 		FREE_ALL();
 		return FALSE;
@@ -678,17 +782,19 @@ bInitDoc(FILE *pFile, off_t tFilesize)
 		break;
 	}
 	bResult = bGetDocumentText(pFile, &PPS_info,
-			aiBBD, iBBDLen, aiSBD, iSBDLen,
+			aiBBD, tBBDLen, aiSBD, tSBDLen,
 			aucHeader, iWordVersion);
 	if (bResult) {
+		vGetDocumentData(pFile, &PPS_info,
+			aiBBD, tBBDLen, aucHeader, iWordVersion);
 		vSetDefaultTabWidth(pFile, &PPS_info,
-			aiBBD, iBBDLen, aiSBD, iSBDLen,
+			aiBBD, tBBDLen, aiSBD, tSBDLen,
 			aucHeader, iWordVersion);
 		vGetParagraphInfo(pFile, &PPS_info,
-			aiBBD, iBBDLen, aiSBD, iSBDLen,
+			aiBBD, tBBDLen, aiSBD, tSBDLen,
 			aucHeader, iWordVersion);
 		vGetNotesInfo(pFile, &PPS_info,
-			aiBBD, iBBDLen, aiSBD, iSBDLen,
+			aiBBD, tBBDLen, aiSBD, tSBDLen,
 			aucHeader, iWordVersion);
 	}
 	FREE_ALL();
@@ -697,22 +803,24 @@ bInitDoc(FILE *pFile, off_t tFilesize)
 
 /*
  * pOpenDocument - open and initialize the document
+ *
+ * Returns the pointer to the document file, or NULL in case of failure
  */
 FILE *
 pOpenDocument(const char *szFilename)
 {
 	FILE	*pFile;
-	off_t	tFilesize;
+	long	lFilesize;
 
-	tFilesize = tGetFilesize(szFilename);
-	if (tFilesize == 0 || tFilesize == (off_t)-1) {
+	lFilesize = lGetFilesize(szFilename);
+	if (lFilesize <= 0) {
 		return NULL;
 	}
 	pFile = pInitFile(szFilename);
 	if (pFile == NULL) {
 		return NULL;
 	}
-	if (bInitDoc(pFile, tFilesize)) {
+	if (bInitDoc(pFile, lFilesize)) {
 		return pFile;
 	}
 	(void)fclose(pFile);
@@ -729,13 +837,13 @@ vCloseDocument(FILE *pFile)
 
 	/* Free the memory */
 	vDestroyTextBlockList();
+	vDestroyDataBlockList();
 	vDestroyRowInfoList();
 	vDestroyStyleInfoList();
 	vDestroyFontInfoList();
+	vDestroyPicInfoList();
 	vDestroyNotesInfoLists();
-#if defined(__riscos)
 	vDestroyFontTable();
-#endif /* __riscos */
 	/* Close the file */
 	(void)fclose(pFile);
 } /* end of vCloseDocument */
@@ -744,13 +852,14 @@ vCloseDocument(FILE *pFile)
  * Common part of the file checking functions
  */
 static BOOL
-bCheckBytes(const char *szFilename, const unsigned char *aucBytes, int iBytes)
+bCheckBytes(const char *szFilename,
+	const unsigned char *aucBytes, size_t tBytes)
 {
 	FILE	*pFile;
 	int	iIndex, iChar;
 	BOOL	bResult;
 
-	fail(aucBytes == NULL || iBytes <= 0);
+	fail(aucBytes == NULL || tBytes == 0);
 
 	if (szFilename == NULL || szFilename[0] == '\0') {
 		return FALSE;
@@ -760,9 +869,11 @@ bCheckBytes(const char *szFilename, const unsigned char *aucBytes, int iBytes)
 		return FALSE;
 	}
 	bResult = TRUE;
-	for (iIndex = 0; iIndex < iBytes; iIndex++) {
+	for (iIndex = 0; iIndex < (int)tBytes; iIndex++) {
 		iChar = getc(pFile);
 		if (iChar == EOF || iChar != (int)aucBytes[iIndex]) {
+			DBG_HEX(iChar);
+			DBG_HEX(aucBytes[iIndex]);
 			bResult = FALSE;
 			break;
 		}
@@ -780,13 +891,16 @@ bIsSupportedWordFile(const char *szFilename)
 {
 	static unsigned char	aucBytes[] =
 		{ 0xd0, 0xcf, 0x11, 0xe0, 0xa1, 0xb1, 0x1a, 0xe1 };
-	off_t	tSize;
+	long	lSize;
 
 	if (szFilename == NULL || szFilename[0] == '\0') {
+		DBG_MSG("No file given");
 		return FALSE;
 	}
-	tSize = tGetFilesize(szFilename);
-	if (tSize < 3 * BIG_BLOCK_SIZE || tSize % BIG_BLOCK_SIZE != 0) {
+	lSize = lGetFilesize(szFilename);
+	if (lSize < 3 * BIG_BLOCK_SIZE || lSize % BIG_BLOCK_SIZE != 0) {
+		DBG_DEC(lSize);
+		DBG_MSG("Size mismatch");
 		return FALSE;
 	}
 	return bCheckBytes(szFilename, aucBytes, elementsof(aucBytes));
@@ -809,7 +923,7 @@ bIsWord245File(const char *szFilename)
 	};
 	int	iIndex;
 
-	for (iIndex = 0; iIndex < elementsof(aucBytes); iIndex++) {
+	for (iIndex = 0; iIndex < (int)elementsof(aucBytes); iIndex++) {
 		if (bCheckBytes(szFilename,
 				aucBytes[iIndex],
 				elementsof(aucBytes[iIndex]))) {
